Package: ArvindSt
Title: Five Novel Stochastic Regression Models with Arvind-Distributed
        Errors and Effects
Version: 1.0.0
Authors@R: c(
    person("Shikhar", "Tyagi",
           email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844")),
    person("Arvind", "Pandey",
           email = "arvindmzu@gmail.com",
           role = "aut")
    )
Description: Implements the 'Arvind' distribution and five novel stochastic
    regression models that replace the traditional Gaussian error assumption
    with 'Arvind'-distributed errors. The 'Arvind' distribution is a flexible
    single-parameter continuous distribution on the positive real line
    characterised by a polynomial numerator with Gaussian-type decay. The
    package provides complete distribution functions (darvind(), parvind(),
    qarvind(), rarvind()), maximum likelihood estimation via
    fit_arvind_mle(), and five model-fitting routines: Random Walk on
    Coefficients via fit_rw1(), Time-Varying Coefficient Linear Model via
    fit_tvlm(), Simulation-Extrapolation via fit_simex(), Mixed-Effects
    Regression via fit_mixed(), and Regime-Switching Hidden Markov Model via
    fit_hmm(). Additionally provides Monte Carlo forecasting with prediction
    intervals via forecast_arvind(), comprehensive goodness-of-fit
    diagnostics (21 metrics and 25 plots) via diagnostics_arvind() and
    plot_arvind(), k-fold and rolling-window cross-validation via
    cv_arvind(), and unified model comparison via summary_arvind().
    For more details see Pandey, Singh, Tyagi, and Tyagi (2024)
    "Modelling climate, COVID-19, and reliability data: A new continuous
    lifetime model under different methods of estimation",
    Statistics and Applications, 22(2),
    <https://ssca.org.in/journal.html>.
License: MIT + file LICENSE
Depends: R (>= 4.0.0)
Imports: stats, graphics, grDevices, utils, ggplot2, forecast, tvReg,
        lme4, depmixS4, reshape2, rlang
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-05 17:16:50 UTC; 30017827
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Arvind Pandey [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 18:20:02 UTC
Built: R 4.5.2; ; 2026-05-11 19:06:20 UTC; unix

Package: ArvindSt
Title: Five Novel Stochastic Regression Models with Arvind-Distributed
        Errors and Effects
Version: 1.1.0
Authors@R: c(
    person("Shikhar", "Tyagi",
           email = "shikhar1093tyagi@gmail.com",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-1606-0844")),
    person("Arvind", "Pandey",
           email = "arvindmzu@gmail.com",
           role = "aut")
    )
Description: Implements the 'Arvind' distribution and five novel stochastic
    regression models that replace the traditional Gaussian error assumption
    with 'Arvind'-distributed errors. The 'Arvind' distribution is a flexible
    single-parameter continuous distribution on the positive real line
    characterised by a polynomial numerator with Gaussian-type decay. The
    package provides complete distribution functions (darvind(), parvind(),
    qarvind(), rarvind()), maximum likelihood estimation via
    fit_arvind_mle(), and five model-fitting routines: Random Walk on
    Coefficients via fit_rw1(), Time-Varying Coefficient Linear Model via
    fit_tvlm(), Simulation-Extrapolation via fit_simex(), Mixed-Effects
    Regression via fit_mixed(), and Regime-Switching Hidden Markov Model via
    fit_hmm(). Additionally provides Monte Carlo forecasting with prediction
    intervals via forecast_arvind(), comprehensive goodness-of-fit
    diagnostics (21 metrics and 25 plots) via diagnostics_arvind() and
    plot_arvind(), k-fold and rolling-window cross-validation via
    cv_arvind(), and unified model comparison via summary_arvind().
    For more details see Pandey, Singh, Tyagi, and Tyagi (2024),
    "Modelling climate, COVID-19, and reliability data: A new continuous
    lifetime model under different methods of estimation",
    'Statistics and Applications', 22(2).
License: MIT + file LICENSE
Depends: R (>= 4.0.0)
Imports: stats, graphics, grDevices, utils, ggplot2, forecast, tvReg,
        lme4, reshape2, rlang
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-06 14:02:12 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Arvind Pandey [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-06 14:30:19 UTC

# Generated by roxygen2: do not edit by hand

export(arvind_mean_fn)
export(arvind_var_fn)
export(cv_arvind)
export(darvind)
export(diagnostics_arvind)
export(fit_arvind_mle)
export(fit_hmm)
export(fit_mixed)
export(fit_rw1)
export(fit_simex)
export(fit_tvlm)
export(forecast_arvind)
export(parvind)
export(plot_arvind)
export(qarvind)
export(rarvind)
export(rarvind_centred)
export(simulate_arvind_data)
export(summary_arvind)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(rlang,.data)
importFrom(stats,Box.test)
importFrom(stats,acf)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,fitted)
importFrom(stats,gaussian)
importFrom(stats,integrate)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,ts)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,tail)

# ArvindSt

<!-- badges: start -->
<!-- badges: end -->

**ArvindSt** is an R package implementing the Arvind distribution and five novel stochastic regression models with Arvind-distributed errors.

## Installation

Install the development version from GitHub:

```r
# install.packages("devtools")
devtools::install_github("shikhartyagi/ArvindSt")
```

## Features

- **Distribution Functions**: `darvind()`, `parvind()`, `qarvind()`, `rarvind()`, `rarvind_centred()`
- **Five Regression Models**:
  - `fit_rw1()` — Random Walk on Coefficients
  - `fit_tvlm()` — Time-Varying Coefficient Linear Model
  - `fit_simex()` — Simulation-Extrapolation
  - `fit_mixed()` — Mixed-Effects Regression
  - `fit_hmm()` — Regime-Switching (HMM)
- **Diagnostics**: `diagnostics_arvind()` (21 metrics), `plot_arvind()` (25 plots)
- **Forecasting**: `forecast_arvind()` with Monte Carlo prediction intervals
- **Cross-Validation**: `cv_arvind()` (k-fold and rolling-window)
- **Model Comparison**: `summary_arvind()`

## Quick Start

```r
library(ArvindSt)

# Load example data
data(climate_consumption)

# Define formula
frm <- Consumption ~ Precip + TempMaxAvg + TempMinAvg + HumidMax + HumidAvg

# Fit all five models
m1 <- fit_rw1(frm, climate_consumption)
m2 <- fit_tvlm(frm, climate_consumption)
m3 <- fit_simex(frm, climate_consumption, me_vars = c("Precip", "TempMaxAvg"))
m4 <- fit_mixed(frm, climate_consumption, group_var = "Season")
m5 <- fit_hmm(frm, climate_consumption, nstates = 2)

# Compare all models
summary_arvind(m1, m2, m3, m4, m5)
```

## Authors

- **Shikhar Tyagi** (maintainer) — shikhar1093tyagi@gmail.com
- **Arvind Pandey** — arvindmzu@gmail.com

## License

MIT

#' Fit Regime-Switching Regression (HMM)
#'
#' Fits a hidden Markov model with state-dependent coefficients and
#' Arvind-distributed errors. The EM algorithm with forward-backward
#' recursions is used for parameter estimation, and the Viterbi algorithm
#' decodes the most likely state sequence.
#'
#' @param formula an object of class \code{\link[stats]{formula}}.
#' @param data a data frame containing the variables in the formula.
#' @param nstates integer; number of hidden states (default: 2).
#' @param seed integer; random seed for reproducibility (default: 42).
#' @param maxiter integer; maximum EM iterations (default: 100).
#' @param tol numeric; convergence tolerance (default: 1e-6).
#'
#' @return An object of class \code{"ArvindFit"}, a list containing the
#'   same standard fields as [fit_rw1()], plus:
#' \describe{
#'   \item{nstates}{integer; number of hidden states.}
#'   \item{states}{integer vector; Viterbi-decoded state sequence.}
#'   \item{trans_probs}{matrix; estimated transition probability matrix.}
#'   \item{state_betas}{list of numeric vectors; state-specific coefficients.}
#'   \item{state_sigmas}{numeric vector; state-specific standard deviations.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m5 <- fit_hmm(Y ~ X1 + X2 + X3, dat, nstates = 2, seed = 42)
#' m5$states
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()], [cv_arvind()]
#'
#' @export
fit_hmm <- function(formula, data,
                    nstates = 2, seed = 42,
                    maxiter = 100, tol = 1e-6) {
  set.seed(seed)
  mf <- model.frame(formula, data)
  Y  <- model.response(mf)
  X  <- model.matrix(formula, data)
  n  <- length(Y)
  p  <- ncol(X)
  K  <- nstates

  # Initialize parameters
  init_fit <- lm(Y ~ X - 1)
  init_beta <- coef(init_fit)
  init_sigma <- summary(init_fit)$sigma

  state_betas <- lapply(seq_len(K), function(k) init_beta + rnorm(p, 0, 0.1))
  state_sigmas <- rep(init_sigma, K)

  # Initialize transition matrix (diagonal dominance)
  trans_probs <- matrix(0.1, nrow = K, ncol = K)
  diag(trans_probs) <- 0.9
  trans_probs <- trans_probs / rowSums(trans_probs)

  # Initialize state probabilities
  init_probs <- rep(1/K, K)

  # EM algorithm
  for (iter in seq_len(maxiter)) {
    old_params <- c(unlist(state_betas), state_sigmas, as.vector(trans_probs))

    # E-step: Forward-backward
    # Compute emission probabilities
    emit_probs <- matrix(0, nrow = n, ncol = K)
    for (k in seq_len(K)) {
      mu <- X %*% state_betas[[k]]
      emit_probs[, k] <- dnorm(Y, mu, state_sigmas[k])
    }

    # Forward pass
    alpha <- matrix(0, nrow = n, ncol = K)
    alpha[1, ] <- init_probs * emit_probs[1, ]
    alpha[1, ] <- alpha[1, ] / sum(alpha[1, ])

    for (t in 2:n) {
      for (j in seq_len(K)) {
        alpha[t, j] <- emit_probs[t, j] * sum(alpha[t-1, ] * trans_probs[, j])
      }
      alpha[t, ] <- alpha[t, ] / sum(alpha[t, ])
    }

    # Backward pass
    beta <- matrix(0, nrow = n, ncol = K)
    beta[n, ] <- 1
    beta[n, ] <- beta[n, ] / sum(beta[n, ])

    for (t in (n-1):1) {
      for (i in seq_len(K)) {
        beta[t, i] <- sum(trans_probs[i, ] * emit_probs[t+1, ] * beta[t+1, ])
      }
      beta[t, ] <- beta[t, ] / sum(beta[t, ])
    }

    # Compute gamma (state probabilities) and xi (transition probabilities)
    gamma <- alpha * beta
    gamma <- gamma / rowSums(gamma)

    xi <- array(0, dim = c(n-1, K, K))
    for (t in 1:(n-1)) {
      for (i in seq_len(K)) {
        for (j in seq_len(K)) {
          xi[t, i, j] <- alpha[t, i] * trans_probs[i, j] *
                          emit_probs[t+1, j] * beta[t+1, j]
        }
      }
      xi[t, , ] <- xi[t, , ] / sum(xi[t, , ])
    }

    # M-step: Update parameters
    # Update transition probabilities
    trans_probs <- matrix(0, nrow = K, ncol = K)
    for (i in seq_len(K)) {
      for (j in seq_len(K)) {
        trans_probs[i, j] <- sum(xi[, i, j]) / sum(gamma[1:(n-1), i])
      }
    }
    # Normalize rows
    trans_probs <- trans_probs / rowSums(trans_probs)

    # Update initial probabilities
    init_probs <- gamma[1, ]

    # Update state-specific parameters
    for (k in seq_len(K)) {
      wk <- gamma[, k]
      if (sum(wk) > 1e-10) {
        X_w <- X * wk
        state_betas[[k]] <- solve(t(X) %*% (X * wk)) %*% t(X_w) %*% Y
        resid_k <- Y - X %*% state_betas[[k]]
        state_sigmas[k] <- sqrt(sum(wk * resid_k^2) / sum(wk))
      }
    }

    # Check convergence
    new_params <- c(unlist(state_betas), state_sigmas, as.vector(trans_probs))
    if (max(abs(new_params - old_params)) < tol) break
  }

  # Viterbi algorithm for state decoding
  viterbi <- matrix(0, nrow = n, ncol = K)
  backptr <- matrix(0, nrow = n, ncol = K)

  viterbi[1, ] <- log(init_probs) + log(emit_probs[1, ])

  for (t in 2:n) {
    for (j in seq_len(K)) {
      max_val <- -Inf
      max_idx <- 1
      for (i in seq_len(K)) {
        val <- viterbi[t-1, i] + log(trans_probs[i, j])
        if (val > max_val) {
          max_val <- val
          max_idx <- i
        }
      }
      viterbi[t, j] <- max_val + log(emit_probs[t, j])
      backptr[t, j] <- max_idx
    }
  }

  # Backtrack
  viterbi_states <- integer(n)
  viterbi_states[n] <- which.max(viterbi[n, ])
  for (t in (n-1):1) {
    viterbi_states[t] <- backptr[t+1, viterbi_states[t+1]]
  }

  # Compute fitted values
  Y_hat <- numeric(n)
  for (i in seq_len(n)) {
    k <- viterbi_states[i]
    Y_hat[i] <- sum(X[i, ] * state_betas[[k]])
  }

  resid_raw <- Y - Y_hat
  arv <- make_arvind_resid(resid_raw, Y)

  result <- list(
    model_type    = "Regime-Switching",
    fitted        = Y_hat,
    residuals     = resid_raw,
    theta         = arv$theta,
    sigma         = arv$sigma,
    shift         = arv$shift,
    e_pos         = arv$e_pos,
    negloglik     = arv$negloglik,
    nstates       = nstates,
    states        = viterbi_states,
    trans_probs   = trans_probs,
    state_betas   = state_betas,
    state_sigmas  = state_sigmas,
    n = n, p = p,
    X = X, Y = Y,
    formula = formula,
    data = data
  )
  class(result) <- "ArvindFit"
  return(result)
}

#' Summary and Comparison of Multiple Arvind Models
#'
#' Accepts multiple \code{ArvindFit} objects, computes diagnostics for each,
#' produces a unified comparison table, and prints the best model by RMSE,
#' R-squared, and AIC.
#'
#' @param ... one or more objects of class \code{"ArvindFit"}.
#' @param comparison_plots logical; if \code{TRUE} (default), generate
#'   comparison plots.
#' @param output_dir character; directory to save comparison plots.
#'   Defaults to a temporary directory.
#'
#' @return A data frame of diagnostic metrics (one row per model) is returned
#'   invisibly.
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' summary_arvind(m1)
#'
#' @seealso [diagnostics_arvind()], [plot_arvind()]
#'
#' @export
summary_arvind <- function(...,
                           comparison_plots = TRUE,
                           output_dir = tempdir()) {
  fits <- list(...)
  comp_df <- do.call(rbind,
    lapply(fits, diagnostics_arvind))

  message("\n=== Model Comparison ===")
  print(comp_df, row.names = FALSE, digits = 4)
  message(sprintf("\n  Best RMSE: %s",
    comp_df$Model[which.min(comp_df$RMSE)]))
  message(sprintf("  Best R2:   %s",
    comp_df$Model[which.max(comp_df$R2)]))
  message(sprintf("  Best AIC:  %s",
    comp_df$Model[which.min(comp_df$AIC)]))

  if (comparison_plots && length(fits) > 1) {
    if (!dir.exists(output_dir))
      dir.create(output_dir, recursive = TRUE)

    theme_paper <- ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        plot.title = ggplot2::element_text(
          face = "bold", hjust = 0.5, size = 14),
        panel.grid.minor = ggplot2::element_blank(),
        legend.position = "bottom")

    # C1: All Models Overlay
    grDevices::png(file.path(output_dir, "C1_All_Models_Overlay.png"),
        width = 1000, height = 600)
    n_max <- max(sapply(fits, function(f) f$n))
    df_all <- do.call(rbind, lapply(fits, function(f) {
      data.frame(
        Index = seq_len(f$n),
        Fitted = f$fitted,
        Model = f$model_type,
        stringsAsFactors = FALSE)
    }))
    df_obs <- data.frame(Index = seq_len(fits[[1]]$n),
                         Y = fits[[1]]$Y)
    p_c1 <- ggplot2::ggplot() +
      ggplot2::geom_line(data = df_obs,
                         ggplot2::aes(x = .data$Index, y = .data$Y),
                         colour = "black", linewidth = 1) +
      ggplot2::geom_line(data = df_all,
                         ggplot2::aes(x = .data$Index, y = .data$Fitted,
                                      colour = .data$Model),
                         linetype = "dashed") +
      ggplot2::labs(title = "All Models - Observed vs Fitted",
                    x = "Time Index", y = "Value", colour = "") +
      theme_paper
    print(p_c1)
    grDevices::dev.off()

    # C3: Error Metrics Bars
    grDevices::png(file.path(output_dir, "C3_Error_Metrics.png"),
        width = 800, height = 600)
    df_metrics <- data.frame(
      Model = comp_df$Model,
      RMSE = comp_df$RMSE,
      MAE = comp_df$MAE,
      stringsAsFactors = FALSE)
    df_long <- reshape2::melt(df_metrics, id.vars = "Model",
                              variable.name = "Metric", value.name = "Value")
    p_c3 <- ggplot2::ggplot(df_long,
                             ggplot2::aes(x = .data$Model, y = .data$Value,
                                          fill = .data$Metric)) +
      ggplot2::geom_bar(stat = "identity", position = "dodge") +
      ggplot2::labs(title = "Error Metrics Comparison",
                    x = "", y = "Value", fill = "") +
      theme_paper
    print(p_c3)
    grDevices::dev.off()

    # C5: Residual Boxplots
    grDevices::png(file.path(output_dir, "C5_Residual_Boxplots.png"),
        width = 800, height = 600)
    df_resid <- do.call(rbind, lapply(fits, function(f) {
      data.frame(Model = f$model_type, Residuals = f$residuals,
                 stringsAsFactors = FALSE)
    }))
    p_c5 <- ggplot2::ggplot(df_resid,
                             ggplot2::aes(x = .data$Model,
                                          y = .data$Residuals,
                                          fill = .data$Model)) +
      ggplot2::geom_boxplot(alpha = 0.7, show.legend = FALSE) +
      ggplot2::labs(title = "Residual Boxplots", x = "", y = "Residuals") +
      theme_paper
    print(p_c5)
    grDevices::dev.off()

    # C8: R2 / Adj R2
    grDevices::png(file.path(output_dir, "C8_R2_Comparison.png"),
        width = 800, height = 600)
    df_r2 <- data.frame(
      Model = comp_df$Model,
      R2 = comp_df$R2,
      AdjR2 = comp_df$AdjR2,
      stringsAsFactors = FALSE)
    df_r2_long <- reshape2::melt(df_r2, id.vars = "Model",
                                  variable.name = "Metric",
                                  value.name = "Value")
    p_c8 <- ggplot2::ggplot(df_r2_long,
                             ggplot2::aes(x = .data$Model, y = .data$Value,
                                          fill = .data$Metric)) +
      ggplot2::geom_bar(stat = "identity", position = "dodge") +
      ggplot2::labs(title = "R-squared Comparison",
                    x = "", y = "Value", fill = "") +
      theme_paper
    print(p_c8)
    grDevices::dev.off()

    message(sprintf("  Comparison plots saved to %s", output_dir))
  }

  invisible(comp_df)
}

test_that("fit_rw1 returns ArvindFit object", {
  dat <- simulate_arvind_data(n = 50, seed = 1)
  m <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)

  expect_s3_class(m, "ArvindFit")
  expect_equal(m$model_type, "RW1-approx")
  expect_length(m$fitted, m$n)
  expect_length(m$residuals, m$n)
  expect_true(is.numeric(m$theta))
  expect_true(m$theta > 0)
})

test_that("diagnostics_arvind returns a data frame with 21 metrics", {
  dat <- simulate_arvind_data(n = 50, seed = 1)
  m <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
  d <- diagnostics_arvind(m)

  expect_s3_class(d, "data.frame")
  expect_equal(nrow(d), 1)
  expect_true("MSE" %in% names(d))
  expect_true("RMSE" %in% names(d))
  expect_true("R2" %in% names(d))
  expect_true("AIC" %in% names(d))
  expect_true("KS_stat" %in% names(d))
  expect_true("Theta" %in% names(d))
})

test_that("cv_arvind returns correct structure", {
  dat <- simulate_arvind_data(n = 50, seed = 1)
  m <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
  cv <- cv_arvind(m, k_folds = 3, rolling = FALSE, seed = 42)

  expect_true(is.list(cv))
  expect_length(cv$cv_rmse, 3)
  expect_true(cv$mean_cv_rmse > 0)
})

test_that("summary_arvind works with single model", {
  dat <- simulate_arvind_data(n = 50, seed = 1)
  m <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
  comp <- summary_arvind(m, comparison_plots = FALSE)

  expect_s3_class(comp, "data.frame")
  expect_equal(nrow(comp), 1)
})

#' Fit Random Walk on Coefficients Model (RW1-approx)
#'
#' Fits a stochastic regression model with time-varying coefficients evolving
#' as a random walk with Arvind-distributed innovations. The observation
#' errors also follow the Arvind distribution.
#'
#' @param formula an object of class \code{\link[stats]{formula}} specifying
#'   the model (e.g., \code{Y ~ X1 + X2}).
#' @param data a data frame containing the variables in the formula.
#' @param theta_innov positive numeric; the Arvind parameter for state
#'   innovations (default: 2.0).
#' @param rw_scale numeric; proportion of OLS coefficients used as innovation
#'   scale (default: 0.01).
#' @param seed integer; random seed for reproducibility (default: 42).
#'
#' @return An object of class \code{"ArvindFit"}, a list containing:
#' \describe{
#'   \item{model_type}{character; \code{"RW1-approx"}.}
#'   \item{fitted}{numeric vector; fitted values.}
#'   \item{residuals}{numeric vector; raw residuals.}
#'   \item{theta}{numeric; estimated Arvind parameter for residuals.}
#'   \item{sigma}{numeric; residual scale.}
#'   \item{shift}{numeric; shift applied to residuals.}
#'   \item{e_pos}{numeric vector; positive standardised residuals.}
#'   \item{negloglik}{numeric; negative log-likelihood.}
#'   \item{beta_t}{matrix; time-varying coefficient paths.}
#'   \item{beta_final}{numeric vector; final coefficient values.}
#'   \item{sigma_rw}{numeric vector; random walk innovation scales.}
#'   \item{theta_innov}{numeric; Arvind parameter used for innovations.}
#'   \item{n}{integer; number of observations.}
#'   \item{p}{integer; number of parameters.}
#'   \item{X}{matrix; design matrix.}
#'   \item{Y}{numeric vector; response variable.}
#'   \item{formula}{the model formula.}
#'   \item{data}{the input data frame.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' m1$theta
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()], [cv_arvind()]
#'
#' @export
fit_rw1 <- function(formula, data,
                    theta_innov = 2.0,
                    rw_scale = 0.01,
                    seed = 42) {
  set.seed(seed)
  mf <- model.frame(formula, data)
  Y  <- model.response(mf)
  X  <- model.matrix(formula, data)
  n  <- length(Y)
  p  <- ncol(X)

  # OLS initialisation
  ols_fit  <- lm(formula, data = data)
  beta_ols <- coef(ols_fit)
  sigma_rw <- rw_scale * abs(beta_ols)

  # State propagation
  beta_t <- matrix(NA, nrow = n, ncol = p)
  Y_hat  <- numeric(n)
  beta_current <- beta_ols

  for (t in seq_len(n)) {
    if (t > 1) {
      w <- rarvind_centred(p, theta_innov)
      beta_current <- beta_current + sigma_rw * w
    }
    beta_t[t, ] <- beta_current
    Y_hat[t]    <- sum(X[t, ] * beta_current)
  }

  # Arvind residual fitting
  resid_raw <- Y - Y_hat
  arv <- make_arvind_resid(resid_raw, Y)

  result <- list(
    model_type  = "RW1-approx",
    fitted      = Y_hat,
    residuals   = resid_raw,
    theta       = arv$theta,
    sigma       = arv$sigma,
    shift       = arv$shift,
    e_pos       = arv$e_pos,
    negloglik   = arv$negloglik,
    beta_t      = beta_t,
    beta_final  = beta_current,
    sigma_rw    = sigma_rw,
    theta_innov = theta_innov,
    n = n, p = p,
    X = X, Y = Y,
    formula = formula,
    data = data
  )
  class(result) <- "ArvindFit"
  return(result)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats acf coef dnorm ecdf fitted lm model.frame model.matrix
#'   model.response optim optimize predict quantile rnorm runif sd
#'   simulate terms ts uniroot var as.formula integrate density gaussian median
#' @importFrom stats Box.test ks.test
#' @importFrom graphics hist lines legend
#' @importFrom grDevices dev.off png
#' @importFrom utils head tail
#' @importFrom rlang .data
## usethis namespace: end
NULL

#' Fit Time-Varying Coefficient Linear Model (tvLM)
#'
#' Fits a time-varying coefficient linear model using kernel-weighted
#' least squares (via the \pkg{tvReg} package) with Arvind-distributed
#' residuals.
#'
#' @param formula an object of class \code{\link[stats]{formula}}.
#' @param data a data frame containing the variables in the formula.
#' @param bw numeric or \code{NULL}; the bandwidth for kernel smoothing.
#'   If \code{NULL} (default), bandwidth is selected automatically via
#'   leave-one-out cross-validation.
#' @param seed integer; random seed for reproducibility (default: 42).
#'
#' @return An object of class \code{"ArvindFit"}, a list containing the
#'   same standard fields as [fit_rw1()], plus:
#' \describe{
#'   \item{tv_coefs}{matrix; time-varying coefficient estimates.}
#'   \item{tv_fit}{the fitted \code{tvReg::tvLM} object.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m2 <- fit_tvlm(Y ~ X1 + X2 + X3, dat, bw = 0.5, seed = 42)
#' m2$theta
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()], [cv_arvind()]
#'
#' @export
fit_tvlm <- function(formula, data,
                     bw = NULL, seed = 42) {
  set.seed(seed)
  mf <- model.frame(formula, data)
  Y  <- model.response(mf)
  X  <- model.matrix(formula, data)
  n  <- length(Y)
  p  <- ncol(X)

  tv_fit <- tryCatch(
    tvReg::tvLM(formula, data = data, bw = bw),
    error = function(e)
      tvReg::tvLM(formula, data = data, bw = 0.5))

  tv_coefs  <- tv_fit$coefficients
  Y_hat     <- rowSums(X * tv_coefs)
  resid_raw <- Y - Y_hat
  arv <- make_arvind_resid(resid_raw, Y)

  result <- list(
    model_type = "tvLM",
    fitted     = Y_hat,
    residuals  = resid_raw,
    theta      = arv$theta,
    sigma      = arv$sigma,
    shift      = arv$shift,
    e_pos      = arv$e_pos,
    negloglik  = arv$negloglik,
    tv_coefs   = tv_coefs,
    tv_fit     = tv_fit,
    n = n, p = p,
    X = X, Y = Y,
    formula = formula,
    data = data
  )
  class(result) <- "ArvindFit"
  return(result)
}

#' Goodness-of-Fit Diagnostics for Arvind Models
#'
#' Computes 21 goodness-of-fit metrics for any fitted \code{ArvindFit}
#' object, including MSE, RMSE, MAE, MAPE, R-squared, AIC, BIC,
#' Kolmogorov-Smirnov test, Anderson-Darling statistic, and more.
#'
#' @param fit an object of class \code{"ArvindFit"} returned by any of the
#'   model-fitting functions.
#'
#' @return A data frame with one row and 21 columns of diagnostics metrics.
#'   See Details for the full list.
#'
#' @details
#' The following metrics are computed:
#' \describe{
#'   \item{Model}{character; the model type.}
#'   \item{MSE}{Mean Squared Error.}
#'   \item{RMSE}{Root Mean Squared Error.}
#'   \item{MAE}{Mean Absolute Error.}
#'   \item{MAPE}{Mean Absolute Percentage Error.}
#'   \item{R2}{R-squared.}
#'   \item{AdjR2}{Adjusted R-squared.}
#'   \item{AIC}{Akaike Information Criterion.}
#'   \item{AICc}{Corrected AIC.}
#'   \item{BIC}{Bayesian Information Criterion.}
#'   \item{LogLik}{Log-likelihood at the MLE.}
#'   \item{Bias}{Mean residual.}
#'   \item{MASE}{Mean Absolute Scaled Error.}
#'   \item{DW}{Durbin-Watson statistic.}
#'   \item{LjungBox_stat}{Ljung-Box test statistic.}
#'   \item{LjungBox_p}{Ljung-Box p-value.}
#'   \item{Theta}{Estimated Arvind parameter.}
#'   \item{KS_stat}{Kolmogorov-Smirnov test statistic.}
#'   \item{KS_pvalue}{Kolmogorov-Smirnov p-value.}
#'   \item{AD_stat}{Anderson-Darling test statistic.}
#'   \item{CvM_stat}{Cramer-von Mises test statistic.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' diagnostics_arvind(m1)
#'
#' @export
diagnostics_arvind <- function(fit) {
  Y <- fit$Y
  Y_hat <- fit$fitted
  resid <- fit$residuals
  theta <- fit$theta
  e_pos <- fit$e_pos
  n <- fit$n
  k <- fit$p + 2
  negloglik <- fit$negloglik

  MSE  <- mean(resid^2)
  RMSE <- sqrt(MSE)
  MAE  <- mean(abs(resid))
  MAPE <- mean(abs(resid / Y)) * 100
  SS_res <- sum(resid^2)
  SS_tot <- sum((Y - mean(Y))^2)
  R2    <- 1 - SS_res / SS_tot
  AdjR2 <- 1 - (1 - R2) * (n - 1) / (n - k - 1)
  ll    <- -negloglik
  AIC_v <- -2 * ll + 2 * k
  AICc  <- AIC_v + (2 * k * (k + 1)) / max(n - k - 1, 1)
  BIC_v <- -2 * ll + k * log(n)
  Bias  <- mean(resid)
  naive_err <- mean(abs(diff(Y)))
  MASE <- MAE / max(naive_err, 1e-10)
  DW   <- sum(diff(resid)^2) / sum(resid^2)
  LB   <- Box.test(resid,
    lag = min(10, n - 1), type = "Ljung-Box")
  KS   <- suppressWarnings(ks.test(e_pos,
    function(x) parvind(x, theta)))

  # Anderson-Darling
  u <- sort(parvind(e_pos, theta))
  m <- length(u)
  u <- pmin(pmax(u, 1e-10), 1 - 1e-10)
  AD <- -m - (1 / m) * sum(
    (2 * (seq_len(m)) - 1) * (log(u) + log(1 - rev(u))))

  # Cramer-von Mises
  CvM <- sum((u - (2 * (seq_len(m)) - 1) / (2 * m))^2) + 1 / (12 * m)

  data.frame(
    Model = fit$model_type,
    MSE = MSE, RMSE = RMSE, MAE = MAE,
    MAPE = MAPE, R2 = R2, AdjR2 = AdjR2,
    AIC = AIC_v, AICc = AICc, BIC = BIC_v,
    LogLik = ll, Bias = Bias, MASE = MASE,
    DW = DW,
    LjungBox_stat = as.numeric(LB$statistic),
    LjungBox_p = LB$p.value,
    Theta = theta,
    KS_stat = as.numeric(KS$statistic),
    KS_pvalue = KS$p.value,
    AD_stat = AD, CvM_stat = CvM,
    stringsAsFactors = FALSE)
}

#' Diagnostic Plots for Arvind Models
#'
#' Generates up to 25 diagnostic plots for a fitted \code{ArvindFit} object,
#' including observed vs fitted, residual histogram with Arvind density overlay,
#' Q-Q plot, ACF, ECDF comparison, and more.
#'
#' @param fit an object of class \code{"ArvindFit"}.
#' @param output_dir character; directory where plots are saved. Defaults to
#'   a temporary directory.
#' @param prefix character or \code{NULL}; prefix for plot filenames. If
#'   \code{NULL}, derived from the model type.
#'
#' @return The \code{fit} object is returned invisibly.
#'
#' @examples
#' \donttest{
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' plot_arvind(m1, output_dir = tempdir())
#' }
#'
#' @export
plot_arvind <- function(fit,
                        output_dir = tempdir(),
                        prefix = NULL) {
  if (is.null(prefix))
    prefix <- gsub("[^A-Za-z0-9]", "_", fit$model_type)

  Y     <- fit$Y
  Y_hat <- fit$fitted
  resid <- fit$residuals
  theta <- fit$theta
  e_pos <- fit$e_pos
  n     <- fit$n

  if (!dir.exists(output_dir))
    dir.create(output_dir, recursive = TRUE)

  theme_paper <- ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(
        face = "bold", hjust = 0.5, size = 14),
      panel.grid.minor = ggplot2::element_blank(),
      legend.position = "bottom")

  # --- Plot 1: Observed vs Fitted ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_01_Obs_vs_Fit.png")),
      width = 800, height = 600)
  df1 <- data.frame(Index = seq_len(n), Observed = Y, Fitted = Y_hat)
  p1 <- ggplot2::ggplot(df1, ggplot2::aes(x = .data$Index)) +
    ggplot2::geom_line(ggplot2::aes(y = .data$Observed, colour = "Observed")) +
    ggplot2::geom_line(ggplot2::aes(y = .data$Fitted, colour = "Fitted"),
                       linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- Observed vs Fitted"),
                  x = "Time Index", y = "Value", colour = "") +
    theme_paper
  print(p1)
  grDevices::dev.off()

  # --- Plot 2: Residual Histogram with Arvind Density ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_02_Resid_Hist.png")),
      width = 800, height = 600)
  x_grid <- seq(min(e_pos) * 0.8, max(e_pos) * 1.2, length.out = 200)
  arvind_dens <- darvind(x_grid, theta)
  df2 <- data.frame(e_pos = e_pos)
  p2 <- ggplot2::ggplot(df2, ggplot2::aes(x = .data$e_pos)) +
    ggplot2::geom_histogram(ggplot2::aes(y = ggplot2::after_stat(density)),
                            bins = 20, fill = "steelblue", alpha = 0.6) +
    ggplot2::geom_line(data = data.frame(x = x_grid, y = arvind_dens),
                       ggplot2::aes(x = .data$x, y = .data$y),
                       colour = "red", linewidth = 1) +
    ggplot2::labs(title = paste(fit$model_type, "- Residual Histogram"),
                  x = "Standardised Residual", y = "Density") +
    theme_paper
  print(p2)
  grDevices::dev.off()

  # --- Plot 3: Q-Q Plot (Arvind) ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_03_QQ_Arvind.png")),
      width = 800, height = 600)
  e_sorted <- sort(e_pos)
  m <- length(e_sorted)
  theo_q <- qarvind((seq_len(m) - 0.5) / m, theta)
  df3 <- data.frame(Theoretical = theo_q, Sample = e_sorted)
  p3 <- ggplot2::ggplot(df3, ggplot2::aes(x = .data$Theoretical,
                                           y = .data$Sample)) +
    ggplot2::geom_point(colour = "steelblue") +
    ggplot2::geom_abline(slope = 1, intercept = 0, colour = "red",
                         linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- Q-Q Plot (Arvind)"),
                  x = "Theoretical Quantiles", y = "Sample Quantiles") +
    theme_paper
  print(p3)
  grDevices::dev.off()

  # --- Plot 4: ACF of Residuals ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_04_ACF.png")),
      width = 800, height = 600)
  acf_vals <- stats::acf(resid, plot = FALSE, lag.max = min(20, n - 1))
  df4 <- data.frame(Lag = acf_vals$lag[-1], ACF = acf_vals$acf[-1])
  p4 <- ggplot2::ggplot(df4, ggplot2::aes(x = .data$Lag, y = .data$ACF)) +
    ggplot2::geom_bar(stat = "identity", fill = "steelblue", width = 0.5) +
    ggplot2::geom_hline(yintercept = c(-1.96, 1.96) / sqrt(n),
                        linetype = "dashed", colour = "red") +
    ggplot2::labs(title = paste(fit$model_type, "- ACF of Residuals"),
                  x = "Lag", y = "ACF") +
    theme_paper
  print(p4)
  grDevices::dev.off()

  # --- Plot 7: Actual vs Predicted Scatter ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_07_Scatter.png")),
      width = 800, height = 600)
  df7 <- data.frame(Actual = Y, Predicted = Y_hat)
  p7 <- ggplot2::ggplot(df7, ggplot2::aes(x = .data$Actual,
                                           y = .data$Predicted)) +
    ggplot2::geom_point(colour = "steelblue") +
    ggplot2::geom_abline(slope = 1, intercept = 0, colour = "red",
                         linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- Actual vs Predicted"),
                  x = "Actual", y = "Predicted") +
    theme_paper
  print(p7)
  grDevices::dev.off()

  # --- Plot 8: Residuals vs Fitted ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_08_Resid_vs_Fit.png")),
      width = 800, height = 600)
  df8 <- data.frame(Fitted = Y_hat, Residuals = resid)
  p8 <- ggplot2::ggplot(df8, ggplot2::aes(x = .data$Fitted,
                                           y = .data$Residuals)) +
    ggplot2::geom_point(colour = "steelblue") +
    ggplot2::geom_hline(yintercept = 0, colour = "red", linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- Residuals vs Fitted"),
                  x = "Fitted Values", y = "Residuals") +
    theme_paper
  print(p8)
  grDevices::dev.off()

  # --- Plot 10: Arvind PDF Family ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_10_PDF_Family.png")),
      width = 800, height = 600)
  thetas <- c(0.5, 1, 2, 5, 10)
  x_seq <- seq(0.01, 5, length.out = 300)
  df10 <- do.call(rbind, lapply(thetas, function(th) {
    data.frame(x = x_seq, y = darvind(x_seq, th),
               theta = paste0("theta = ", th))
  }))
  p10 <- ggplot2::ggplot(df10, ggplot2::aes(x = .data$x, y = .data$y,
                                              colour = .data$theta)) +
    ggplot2::geom_line(linewidth = 0.8) +
    ggplot2::labs(title = "Arvind Distribution PDF Family",
                  x = "x", y = "f(x)", colour = "") +
    theme_paper
  print(p10)
  grDevices::dev.off()

  # --- Plot 11: Arvind CDF Family ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_11_CDF_Family.png")),
      width = 800, height = 600)
  df11 <- do.call(rbind, lapply(thetas, function(th) {
    data.frame(x = x_seq, y = parvind(x_seq, th),
               theta = paste0("theta = ", th))
  }))
  p11 <- ggplot2::ggplot(df11, ggplot2::aes(x = .data$x, y = .data$y,
                                              colour = .data$theta)) +
    ggplot2::geom_line(linewidth = 0.8) +
    ggplot2::labs(title = "Arvind Distribution CDF Family",
                  x = "x", y = "F(x)", colour = "") +
    theme_paper
  print(p11)
  grDevices::dev.off()

  # --- Plot 13: Residual Time Series ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_13_Resid_TS.png")),
      width = 800, height = 600)
  df13 <- data.frame(Index = seq_len(n), Residuals = resid)
  p13 <- ggplot2::ggplot(df13, ggplot2::aes(x = .data$Index,
                                              y = .data$Residuals)) +
    ggplot2::geom_line(colour = "steelblue") +
    ggplot2::geom_hline(yintercept = 0, colour = "red", linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- Residual Time Series"),
                  x = "Time Index", y = "Residuals") +
    theme_paper
  print(p13)
  grDevices::dev.off()

  # --- Plot 14: ECDF vs Arvind CDF ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_14_ECDF.png")),
      width = 800, height = 600)
  ecdf_vals <- stats::ecdf(e_pos)
  x_grid14 <- seq(min(e_pos), max(e_pos), length.out = 200)
  df14 <- data.frame(x = x_grid14,
                     ECDF = ecdf_vals(x_grid14),
                     Arvind = parvind(x_grid14, theta))
  p14 <- ggplot2::ggplot(df14, ggplot2::aes(x = .data$x)) +
    ggplot2::geom_step(ggplot2::aes(y = .data$ECDF, colour = "ECDF")) +
    ggplot2::geom_line(ggplot2::aes(y = .data$Arvind, colour = "Arvind CDF")) +
    ggplot2::labs(title = paste(fit$model_type, "- ECDF vs Arvind CDF"),
                  x = "x", y = "Probability", colour = "") +
    theme_paper
  print(p14)
  grDevices::dev.off()

  # --- Plot 19: P-P Plot ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_19_PP_Plot.png")),
      width = 800, height = 600)
  e_sorted19 <- sort(e_pos)
  m19 <- length(e_sorted19)
  empirical_p <- (seq_len(m19) - 0.5) / m19
  theoretical_p <- parvind(e_sorted19, theta)
  df19 <- data.frame(Theoretical = theoretical_p, Empirical = empirical_p)
  p19 <- ggplot2::ggplot(df19, ggplot2::aes(x = .data$Theoretical,
                                              y = .data$Empirical)) +
    ggplot2::geom_point(colour = "steelblue") +
    ggplot2::geom_abline(slope = 1, intercept = 0, colour = "red",
                         linetype = "dashed") +
    ggplot2::labs(title = paste(fit$model_type, "- P-P Plot"),
                  x = "Theoretical CDF", y = "Empirical CDF") +
    theme_paper
  print(p19)
  grDevices::dev.off()

  # --- Plot 21: Scale-Location ---
  grDevices::png(file.path(output_dir, paste0(prefix, "_21_Scale_Location.png")),
      width = 800, height = 600)
  df21 <- data.frame(Fitted = Y_hat,
                     SqrtAbsResid = sqrt(abs(resid)))
  p21 <- ggplot2::ggplot(df21, ggplot2::aes(x = .data$Fitted,
                                              y = .data$SqrtAbsResid)) +
    ggplot2::geom_point(colour = "steelblue") +
    ggplot2::geom_smooth(method = "loess", colour = "red", se = FALSE) +
    ggplot2::labs(title = paste(fit$model_type, "- Scale-Location"),
                  x = "Fitted Values", y = "sqrt(|Residuals|)") +
    theme_paper
  print(p21)
  grDevices::dev.off()

  message(sprintf("  %s: Diagnostic plots saved to %s",
              fit$model_type, output_dir))
  invisible(fit)
}

#' Fit Mixed-Effects Regression with Arvind Errors
#'
#' Fits a mixed-effects regression model with Arvind-distributed random
#' effects and observation-level errors. Estimation uses a two-stage approach:
#' REML initialisation via \pkg{lme4}, followed by Arvind MLE on the
#' residuals.
#'
#' @param formula an object of class \code{\link[stats]{formula}} specifying
#'   the fixed-effects structure.
#' @param data a data frame containing the variables in the formula and the
#'   grouping variable.
#' @param group_var character string; the name of the grouping variable in
#'   \code{data} (default: \code{"Season"}).
#' @param re_formula optional random-effects formula (e.g.,
#'   \code{(1 + X1 | group)}). If \code{NULL} (default), a random intercept
#'   model \code{(1 | group_var)} is used.
#' @param seed integer; random seed for reproducibility (default: 42).
#'
#' @return An object of class \code{"ArvindFit"}, a list containing the
#'   same standard fields as [fit_rw1()], plus:
#' \describe{
#'   \item{lme_model}{the fitted \code{lme4::lmer} object.}
#'   \item{theta_re}{numeric; Arvind parameter estimated from random effects.}
#'   \item{group_var}{character; the grouping variable name.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m4 <- fit_mixed(Y ~ X1 + X2 + X3, dat, group_var = "Group", seed = 42)
#' m4$theta
#' m4$theta
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()], [cv_arvind()]
#'
#' @export
fit_mixed <- function(formula, data,
                      group_var = "Season",
                      re_formula = NULL,
                      seed = 42) {
  set.seed(seed)
  mf <- model.frame(formula, data)
  Y  <- model.response(mf)
  X  <- model.matrix(formula, data)
  n  <- length(Y)
  p  <- ncol(X)

  resp      <- as.character(formula[[2]])
  fixed_rhs <- as.character(formula[[3]])
  if (is.null(re_formula)) {
    full_frm <- as.formula(paste0(
      resp, " ~ ", fixed_rhs,
      " + (1|", group_var, ")"))
  } else {
    full_frm <- as.formula(paste0(
      resp, " ~ ", fixed_rhs,
      " + ", deparse(re_formula)))
  }

  me_fit <- tryCatch(
    lme4::lmer(full_frm, data = data, REML = FALSE),
    error = function(e) {
      full_frm2 <- as.formula(paste0(
        resp, " ~ ", fixed_rhs,
        " + (1|", group_var, ")"))
      lme4::lmer(full_frm2, data = data, REML = FALSE)
    })

  Y_hat     <- fitted(me_fit)
  resid_raw <- Y - Y_hat
  arv <- make_arvind_resid(resid_raw, Y)

  # Fit Arvind to random effects
  re_vals <- unlist(lme4::ranef(me_fit))
  re_pos  <- re_vals - min(re_vals) + 0.01
  arv_re  <- fit_arvind_mle(re_pos)

  result <- list(
    model_type  = "Mixed-Effects",
    fitted      = Y_hat,
    residuals   = resid_raw,
    theta       = arv$theta,
    sigma       = arv$sigma,
    shift       = arv$shift,
    e_pos       = arv$e_pos,
    negloglik   = arv$negloglik,
    lme_model   = me_fit,
    theta_re    = arv_re$theta,
    group_var   = group_var,
    formula_full = full_frm,
    n = n, p = p,
    X = X, Y = Y,
    formula = formula,
    data = data
  )
  class(result) <- "ArvindFit"
  return(result)
}

#' Arvind Distribution Density Function
#'
#' Computes the probability density function (PDF) of the Arvind distribution.
#'
#' The Arvind distribution with parameter \eqn{\theta > 0} has PDF
#' \deqn{f(x; \theta) = \frac{\theta(1 + 2x + 2\theta x^2)}{(1 + \theta x)^2}
#'   \exp(-\theta x^2), \quad x > 0.}
#'
#' @param x numeric vector of quantiles.
#' @param theta positive numeric scalar; the distribution parameter.
#' @param log logical; if \code{TRUE}, log-density is returned. Default
#'   \code{FALSE}.
#'
#' @return A numeric vector of density values (or log-density values when
#'   \code{log = TRUE}).
#'
#' @examples
#' # Evaluate the PDF at several points
#' darvind(c(0.5, 1, 2), theta = 1)
#'
#' # Log-density
#' darvind(1, theta = 2, log = TRUE)
#'
#' # Returns 0 for x <= 0
#' darvind(-1, theta = 1)
#'
#' @export
darvind <- function(x, theta, log = FALSE) {
  val <- rep(-Inf, length(x))
  idx <- (x > 0) & (theta > 0)
  if (any(idx)) {
    xx <- x[idx]
    th <- theta
    num <- th * (1 + 2 * xx + 2 * th * xx^2)
    den <- (1 + th * xx)^2
    lv  <- log(num) - log(den) - th * xx^2
    val[idx] <- lv
  }
  if (log) return(val) else return(exp(val))
}

#' Arvind Distribution Function (CDF)
#'
#' Computes the cumulative distribution function (CDF) of the Arvind
#' distribution.
#'
#' The CDF is given by
#' \deqn{F(x; \theta) = 1 - \frac{1}{1 + \theta x} \exp(-\theta x^2),
#'   \quad x > 0.}
#'
#' @param q numeric vector of quantiles.
#' @param theta positive numeric scalar; the distribution parameter.
#' @param lower.tail logical; if \code{TRUE} (default), probabilities are
#'   \eqn{P(X \le q)}{P(X <= q)}; otherwise \eqn{P(X > q)}.
#'
#' @return A numeric vector of probabilities.
#'
#' @examples
#' parvind(1, theta = 1)
#' parvind(c(0.5, 1, 2), theta = 2)
#' parvind(1, theta = 1, lower.tail = FALSE)
#'
#' @export
parvind <- function(q, theta, lower.tail = TRUE) {
  val <- rep(0, length(q))
  idx <- (q > 0) & (theta > 0)
  if (any(idx)) {
    qq <- q[idx]
    th <- theta
    surv <- exp(-th * qq^2) / (1 + th * qq)
    val[idx] <- 1 - surv
  }
  if (lower.tail) return(val) else return(1 - val)
}

#' Arvind Distribution Quantile Function
#'
#' Computes quantiles of the Arvind distribution by numerical inversion
#' of the CDF using \code{\link[stats]{uniroot}}.
#'
#' @param p numeric vector of probabilities (\eqn{0 \le p \le 1}).
#' @param theta positive numeric scalar; the distribution parameter.
#'
#' @return A numeric vector of quantiles.
#'
#' @examples
#' qarvind(0.5, theta = 1)
#' qarvind(c(0.25, 0.5, 0.75), theta = 2)
#'
#' @export
qarvind <- function(p, theta) {
  sapply(p, function(pp) {
    if (pp <= 0) return(0)
    if (pp >= 1) return(Inf)
    fn <- function(x) parvind(x, theta) - pp
    tryCatch(
      uniroot(fn, lower = 1e-10,
              upper = 100 / theta, tol = 1e-12)$root,
      error = function(e) {
        tryCatch(
          uniroot(fn, lower = 1e-10,
                  upper = 1000 / theta,
                  tol = 1e-10)$root,
          error = function(e2) NA_real_)
      })
  })
}

#' Random Generation from the Arvind Distribution
#'
#' Generates random variates from the Arvind distribution using a rejection
#' sampling algorithm with a half-normal proposal distribution.
#'
#' @param n positive integer; number of random variates to generate.
#' @param theta positive numeric scalar; the distribution parameter.
#'
#' @return A numeric vector of length \code{n} containing positive random
#'   variates.
#'
#' @examples
#' set.seed(42)
#' x <- rarvind(100, theta = 1)
#' summary(x)
#'
#' @export
rarvind <- function(n, theta) {
  samples <- numeric(0)
  mode_x <- tryCatch(
    optimize(function(x) darvind(x, theta),
             interval = c(0, 10 / sqrt(theta)),
             maximum = TRUE)$maximum,
    error = function(e) 0.5 / sqrt(theta))
  f_max   <- darvind(mode_x, theta) * 1.5
  prop_sd <- max(1 / sqrt(theta), 0.1)
  while (length(samples) < n) {
    needed     <- (n - length(samples)) * 3
    candidates <- abs(rnorm(needed, 0, prop_sd))
    f_vals <- darvind(candidates, theta)
    g_vals <- f_max * dnorm(candidates, 0, prop_sd) *
              2 / dnorm(0, 0, prop_sd)
    g_vals <- pmax(g_vals,
                   f_max * exp(-theta * candidates^2))
    u      <- runif(needed)
    accept <- u <= f_vals / pmax(g_vals, f_vals * 1.01)
    samples <- c(samples, candidates[accept])
  }
  return(samples[1:n])
}

#' Centred Random Generation from the Arvind Distribution
#'
#' Generates centred Arvind variates with approximately zero mean, suitable
#' for use as error terms and innovation terms in stochastic regression models.
#'
#' The centred variate is computed as \eqn{\tilde{\varepsilon} =
#' \varepsilon - \mu_A(\theta)}, where \eqn{\varepsilon \sim
#' \mathrm{Arvind}(\theta)} and \eqn{\mu_A(\theta)} is the mean of the
#' Arvind distribution.
#'
#' @param n positive integer; number of random variates to generate.
#' @param theta positive numeric scalar; the distribution parameter.
#'
#' @return A numeric vector of length \code{n} with approximately zero mean.
#'
#' @examples
#' set.seed(42)
#' eps <- rarvind_centred(1000, theta = 2)
#' mean(eps)  # approximately 0
#'
#' @export
rarvind_centred <- function(n, theta) {
  raw  <- rarvind(n, theta)
  mu_a <- tryCatch(
    integrate(function(x) x * darvind(x, theta),
              0, Inf)$value,
    error = function(e) mean(raw))
  return(raw - mu_a)
}

test_that("darvind handles vector theta edge case", {
  # Single theta applied to vector x
  vals <- darvind(c(0.5, 1, 2), theta = 1)
  expect_length(vals, 3)
  expect_true(all(vals > 0))
})

test_that("parvind returns values in [0, 1]", {
  x_seq <- c(-1, 0, 0.001, 0.5, 1, 5, 50)
  vals <- parvind(x_seq, theta = 1)
  expect_true(all(vals >= 0))
  expect_true(all(vals <= 1))
})

test_that("Arvind survival function equals 1 - CDF", {
  x <- 1.5
  theta <- 3
  cdf <- parvind(x, theta)
  surv <- exp(-theta * x^2) / (1 + theta * x)
  expect_equal(cdf, 1 - surv, tolerance = 1e-10)
})

test_that("Hazard rate is positive", {
  theta <- 2
  x_seq <- seq(0.01, 2, length.out = 100)
  # h(x) = f(x) / S(x)
  f_vals <- darvind(x_seq, theta)
  s_vals <- parvind(x_seq, theta, lower.tail = FALSE)
  h_vals <- f_vals / s_vals
  
  expect_true(all(h_vals > 0))
  expect_true(all(!is.na(h_vals)))
})

test_that("rarvind generates n positive values", {
  set.seed(42)
  x <- rarvind(100, theta = 2)
  expect_length(x, 100)
  expect_true(all(x > 0))
})

test_that("rarvind sample mean is close to theoretical mean", {
  set.seed(42)
  theta <- 2
  x <- rarvind(5000, theta)
  mu_th <- arvind_mean_fn(theta)
  expect_equal(mean(x), mu_th, tolerance = 0.1)
})

test_that("rarvind_centred has mean approximately 0", {
  set.seed(42)
  eps <- rarvind_centred(5000, theta = 2)
  expect_length(eps, 5000)
  expect_equal(mean(eps), 0, tolerance = 0.1)
})

test_that("rarvind works for different theta values", {
  set.seed(42)
  for (th in c(0.5, 1, 5, 10)) {
    x <- rarvind(50, th)
    expect_length(x, 50)
    expect_true(all(x > 0))
  }
})

test_that("parvind returns 0 at x = 0", {
  expect_equal(parvind(0, theta = 1), 0)
  expect_equal(parvind(0, theta = 5), 0)
})

test_that("parvind approaches 1 for large x", {
  val <- parvind(100, theta = 1)
  expect_true(val > 0.999)
})

test_that("parvind is monotone increasing", {
  x_seq <- seq(0.01, 10, length.out = 100)
  cdf_vals <- parvind(x_seq, theta = 2)
  diffs <- diff(cdf_vals)
  expect_true(all(diffs >= 0))
})

test_that("parvind lower.tail=FALSE is complement", {
  x <- 1.5
  theta <- 2
  lower <- parvind(x, theta, lower.tail = TRUE)
  upper <- parvind(x, theta, lower.tail = FALSE)
  expect_equal(lower + upper, 1, tolerance = 1e-12)
})

test_that("parvind is consistent with darvind via numerical integration", {
  theta <- 2
  x <- 1.5
  cdf_numerical <- integrate(function(t) darvind(t, theta),
                              lower = 0, upper = x)$value
  cdf_analytical <- parvind(x, theta)
  expect_equal(cdf_numerical, cdf_analytical, tolerance = 1e-6)
})

test_that("qarvind inverts parvind", {
  theta <- 2
  for (p in c(0.1, 0.25, 0.5, 0.75, 0.9)) {
    q <- qarvind(p, theta)
    p_back <- parvind(q, theta)
    expect_equal(p_back, p, tolerance = 1e-6)
  }
})

test_that("qarvind handles boundary values", {
  expect_equal(qarvind(0, theta = 1), 0)
  expect_equal(qarvind(1, theta = 1), Inf)
})

test_that("darvind returns correct values for positive x and theta", {
  # PDF should be positive for x > 0
  val <- darvind(1, theta = 1)
  expect_true(val > 0)

  # Vectorised input
  vals <- darvind(c(0.1, 0.5, 1, 2, 5), theta = 2)
  expect_length(vals, 5)
  expect_true(all(vals > 0))
})

test_that("darvind returns 0 for non-positive x", {
  expect_equal(darvind(0, theta = 1), 0)
  expect_equal(darvind(-1, theta = 1), 0)
  expect_equal(darvind(-5, theta = 2), 0)
})

test_that("darvind log=TRUE gives log of density", {
  x <- 1.5
  theta <- 2
  d_val <- darvind(x, theta, log = FALSE)
  ld_val <- darvind(x, theta, log = TRUE)
  expect_equal(log(d_val), ld_val, tolerance = 1e-12)
})

test_that("darvind PDF integrates to 1", {
  for (th in c(0.5, 1, 2, 5)) {
    integral <- integrate(function(x) darvind(x, th),
                          lower = 0, upper = Inf,
                          subdivisions = 2000)$value
    expect_equal(integral, 1, tolerance = 1e-6)
  }
})

test_that("darvind returns valid probabilities for a range of x", {
  theta <- 2
  x_seq <- seq(0, 5, length.out = 500)
  d_vals <- darvind(x_seq, theta)
  
  expect_true(all(!is.na(d_vals)))
  expect_true(all(d_vals >= 0))
})

#' Generate Simulated Data for Examples
#'
#' Creates a small simulated dataset that mimics the structure needed for
#' demonstrating the \pkg{ArvindSt} model-fitting functions. Useful for
#' examples and testing.
#'
#' @param n integer; number of observations to generate (default: 60).
#' @param seed integer; random seed for reproducibility (default: 42).
#'
#' @return A data frame with columns:
#' \describe{
#'   \item{Y}{numeric; simulated response variable.}
#'   \item{X1}{numeric; first covariate.}
#'   \item{X2}{numeric; second covariate.}
#'   \item{X3}{numeric; third covariate.}
#'   \item{Group}{factor; grouping variable with 4 levels.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' head(dat)
#'
#' @export
simulate_arvind_data <- function(n = 60, seed = 42) {
  set.seed(seed)
  X1 <- rnorm(n, 50, 10)
  X2 <- rnorm(n, 30, 5)
  X3 <- rnorm(n, 20, 8)
  eps <- rarvind(n, theta = 2)
  mu_eps <- tryCatch(
    integrate(function(x) x * darvind(x, 2), 0, Inf)$value,
    error = function(e) mean(eps))
  Y <- 100 + 2 * X1 - 1.5 * X2 + 0.8 * X3 + 10 * (eps - mu_eps)
  Group <- factor(rep(c("A", "B", "C", "D"), length.out = n))
  data.frame(Y = Y, X1 = X1, X2 = X2, X3 = X3, Group = Group)
}

#' Maximum Likelihood Estimation for the Arvind Distribution
#'
#' Fits the Arvind distribution to a vector of positive observations by
#' maximum likelihood. Optimisation is performed on the log-scale via the
#' Brent method.
#'
#' @param e_pos numeric vector of strictly positive observations.
#'
#' @return A list with components:
#' \describe{
#'   \item{theta}{numeric; the MLE of theta.}
#'   \item{negloglik}{numeric; the minimised negative log-likelihood.}
#' }
#'
#' @examples
#' set.seed(42)
#' x <- rarvind(200, theta = 2)
#' fit_arvind_mle(x)
#'
#' @export
fit_arvind_mle <- function(e_pos) {
  if (length(e_pos) < 3 || any(!is.finite(e_pos)))
    return(list(theta = NA, negloglik = NA))
  neg_ll <- function(log_th) {
    th <- exp(log_th)
    ne <- length(e_pos)
    ll <- ne * log(th) +
      sum(log(pmax(1e-300,
                   1 + 2 * e_pos + 2 * th * e_pos^2))) -
      2 * sum(log(1 + th * e_pos)) - th * sum(e_pos^2)
    if (!is.finite(ll)) return(1e15)
    return(-ll)
  }
  opt <- tryCatch(
    optim(log(1 / mean(e_pos)), neg_ll,
          method = "Brent", lower = -10, upper = 15),
    error = function(e)
      list(par = 0, value = 1e15))
  list(theta = exp(opt$par), negloglik = opt$value)
}

#' Transform Residuals for Arvind Fitting
#'
#' Transforms raw residuals to positive values suitable for fitting the Arvind
#' distribution by shifting and standardising.
#'
#' @param resid_raw numeric vector of raw residuals.
#' @param Y_ref numeric vector of observed response values (used for scaling).
#'
#' @return A list with components:
#' \describe{
#'   \item{shift}{numeric; the shift applied.}
#'   \item{sigma}{numeric; the standard deviation used for standardisation.}
#'   \item{e_pos}{numeric vector; positive standardised residuals.}
#'   \item{theta}{numeric; MLE of the Arvind parameter.}
#'   \item{negloglik}{numeric; negative log-likelihood at the MLE.}
#' }
#'
#' @keywords internal
make_arvind_resid <- function(resid_raw, Y_ref) {
  shift_val <- min(resid_raw) - 0.01 * sd(Y_ref)
  resid_pos <- resid_raw - shift_val
  sigma_s   <- sd(resid_pos)
  e_std     <- resid_pos / sigma_s
  e_pos     <- e_std[e_std > 0]
  arv       <- fit_arvind_mle(e_pos)
  list(shift = shift_val, sigma = sigma_s,
       e_pos = e_pos,
       theta = arv$theta, negloglik = arv$negloglik)
}

#' Mean of the Arvind Distribution
#'
#' Computes the theoretical mean of the Arvind distribution with parameter
#' \code{theta} by numerical integration.
#'
#' @param theta positive numeric scalar; the distribution parameter.
#'
#' @return A numeric scalar giving the theoretical mean, or \code{NA} if
#'   integration fails.
#'
#' @examples
#' arvind_mean_fn(1)
#' arvind_mean_fn(2)
#'
#' @export
arvind_mean_fn <- function(theta) {
  tryCatch(integrate(function(x) x * darvind(x, theta),
           0, Inf, subdivisions = 2000,
           rel.tol = 1e-10)$value,
           error = function(e) NA)
}

#' Variance of the Arvind Distribution
#'
#' Computes the theoretical variance of the Arvind distribution with parameter
#' \code{theta} by numerical integration.
#'
#' @param theta positive numeric scalar; the distribution parameter.
#'
#' @return A numeric scalar giving the theoretical variance, or \code{NA} if
#'   integration fails.
#'
#' @examples
#' arvind_var_fn(1)
#' arvind_var_fn(2)
#'
#' @export
arvind_var_fn <- function(theta) {
  mu  <- arvind_mean_fn(theta)
  mu2 <- tryCatch(
    integrate(function(x) x^2 * darvind(x, theta),
              0, Inf, subdivisions = 2000,
              rel.tol = 1e-10)$value,
    error = function(e) NA)
  if (is.na(mu) || is.na(mu2)) return(NA)
  return(mu2 - mu^2)
}

test_that("fit_arvind_mle recovers theta from simulated data", {
  set.seed(42)
  true_theta <- 2
  x <- rarvind(500, true_theta)
  fit <- fit_arvind_mle(x)
  expect_false(is.na(fit$theta))
  expect_equal(fit$theta, true_theta, tolerance = 0.5)
  expect_true(is.finite(fit$negloglik))
})

test_that("fit_arvind_mle returns NA for insufficient data", {
  fit <- fit_arvind_mle(c(1, 2))  # only 2 observations
  expect_true(is.na(fit$theta))
})

test_that("fit_arvind_mle returns NA for non-finite data", {
  fit <- fit_arvind_mle(c(1, 2, Inf, 4))
  expect_true(is.na(fit$theta))
})

test_that("arvind_mean_fn returns positive values", {
  for (th in c(0.5, 1, 2, 5)) {
    mu <- arvind_mean_fn(th)
    expect_true(!is.na(mu))
    expect_true(mu > 0)
  }
})

test_that("arvind_var_fn returns positive values", {
  for (th in c(0.5, 1, 2, 5)) {
    v <- arvind_var_fn(th)
    expect_true(!is.na(v))
    expect_true(v > 0)
  }
})

test_that("arvind_mean_fn decreases with theta", {
  means <- sapply(c(0.5, 1, 2, 5), arvind_mean_fn)
  # Mean should generally decrease as theta increases
  expect_true(means[1] > means[4])
})

#' Fit Simulation-Extrapolation (SIMEX) Model
#'
#' Fits a regression model correcting for measurement error attenuation using
#' the SIMEX algorithm with Arvind-distributed measurement noise and residuals.
#'
#' @param formula an object of class \code{\link[stats]{formula}}.
#' @param data a data frame containing the variables in the formula.
#' @param me_vars character vector of covariate names measured with error.
#'   If \code{NULL} (default), the first two term labels are used.
#' @param me_frac numeric; fraction of marginal variance used as measurement
#'   error variance (default: 0.05).
#' @param lambda_grid numeric vector; SIMEX lambda grid
#'   (default: \code{c(0.5, 1, 1.5, 2)}).
#' @param n_sim integer; number of SIMEX simulation replicates (default: 100).
#' @param theta_me positive numeric; Arvind parameter for measurement error
#'   (default: 2.0).
#' @param seed integer; random seed for reproducibility (default: 123).
#'
#' @return An object of class \code{"ArvindFit"}, a list containing the
#'   same standard fields as [fit_rw1()], plus:
#' \describe{
#'   \item{beta}{numeric vector; SIMEX-corrected coefficient estimates.}
#'   \item{simex_coefs}{matrix; coefficient estimates at each lambda level.}
#'   \item{lambda_grid}{numeric vector; the SIMEX lambda grid used.}
#'   \item{me_vars}{character vector; covariate names with measurement error.}
#'   \item{sigma2_me}{named numeric vector; measurement error variances.}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m3 <- fit_simex(Y ~ X1 + X2 + X3, dat,
#'                 me_vars = c("X1", "X2"),
#'                 n_sim = 20, seed = 123)
#' m3$beta
#' m3$beta
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()], [cv_arvind()]
#'
#' @export
fit_simex <- function(formula, data,
                      me_vars = NULL,
                      me_frac = 0.05,
                      lambda_grid = c(0.5, 1, 1.5, 2),
                      n_sim = 100,
                      theta_me = 2.0,
                      seed = 123) {
  set.seed(seed)
  mf <- model.frame(formula, data)
  Y  <- model.response(mf)
  X  <- model.matrix(formula, data)
  n  <- length(Y)
  p  <- ncol(X)

  if (is.null(me_vars)) {
    cov_names <- attr(terms(formula), "term.labels")
    me_vars   <- cov_names[seq_len(min(2, length(cov_names)))]
  }
  sigma2_me <- sapply(me_vars,
    function(v) var(data[[v]]) * me_frac)

  # Simulation step
  simex_coefs <- matrix(NA, nrow = length(lambda_grid), ncol = p)
  colnames(simex_coefs) <- colnames(X)

  for (l_idx in seq_along(lambda_grid)) {
    lam <- lambda_grid[l_idx]
    coef_sims <- matrix(0, nrow = n_sim, ncol = p)
    for (s in seq_len(n_sim)) {
      data_noisy <- data
      for (v in me_vars) {
        noise <- rarvind_centred(n, theta_me) *
                 sqrt(lam * sigma2_me[v])
        data_noisy[[v]] <- data[[v]] + noise
      }
      fit_s <- lm(formula, data = data_noisy)
      coef_sims[s, ] <- coef(fit_s)
    }
    simex_coefs[l_idx, ] <- colMeans(coef_sims)
  }

  # Extrapolation step
  beta_simex <- numeric(p)
  for (j in seq_len(p)) {
    qfit <- lm(simex_coefs[, j] ~
      lambda_grid + I(lambda_grid^2))
    beta_simex[j] <- predict(qfit,
      newdata = data.frame(lambda_grid = -1))
  }
  names(beta_simex) <- colnames(X)

  Y_hat <- as.numeric(X %*% beta_simex)
  resid_raw <- Y - Y_hat
  arv <- make_arvind_resid(resid_raw, Y)

  result <- list(
    model_type   = "SIMEX",
    fitted       = Y_hat,
    residuals    = resid_raw,
    theta        = arv$theta,
    sigma        = arv$sigma,
    shift        = arv$shift,
    e_pos        = arv$e_pos,
    negloglik    = arv$negloglik,
    beta         = beta_simex,
    simex_coefs  = simex_coefs,
    lambda_grid  = lambda_grid,
    me_vars      = me_vars,
    sigma2_me    = sigma2_me,
    n = n, p = p,
    X = X, Y = Y,
    formula = formula,
    data = data
  )
  class(result) <- "ArvindFit"
  return(result)
}

#' K-Fold and Rolling-Window Cross-Validation
#'
#' Performs k-fold cross-validation and optionally rolling-window
#' (expanding-window) cross-validation for an \code{ArvindFit} model.
#'
#' @param fit an object of class \code{"ArvindFit"}.
#' @param k_folds integer; number of cross-validation folds (default: 5).
#' @param rolling logical; if \code{TRUE} (default), also performs
#'   rolling-window cross-validation.
#' @param n0_frac numeric; fraction of data used as initial training window
#'   for rolling CV (default: 0.5).
#' @param seed integer; random seed for reproducibility (default: 42).
#'
#' @return A list with components:
#' \describe{
#'   \item{cv_rmse}{numeric vector of length \code{k_folds}; per-fold RMSE.}
#'   \item{cv_mae}{numeric vector of length \code{k_folds}; per-fold MAE.}
#'   \item{mean_cv_rmse}{numeric; average k-fold RMSE.}
#'   \item{mean_cv_mae}{numeric; average k-fold MAE.}
#'   \item{roll_rmse}{numeric; rolling-window RMSE (or \code{NA}).}
#' }
#'
#' @examples
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' cv <- cv_arvind(m1, k_folds = 3, rolling = FALSE, seed = 42)
#' cv$mean_cv_rmse
#'
#' @seealso [diagnostics_arvind()], [forecast_arvind()]
#'
#' @export
cv_arvind <- function(fit, k_folds = 5,
                      rolling = TRUE,
                      n0_frac = 0.5,
                      seed = 42) {
  set.seed(seed)
  Y <- fit$Y
  X <- fit$X
  n <- fit$n
  fold_ids <- sample(rep(seq_len(k_folds), length.out = n))
  cv_rmse <- cv_mae <- numeric(k_folds)

  for (f in seq_len(k_folds)) {
    test_idx  <- which(fold_ids == f)
    train_idx <- which(fold_ids != f)

    if (fit$model_type == "tvLM") {
      tv_cv <- tryCatch(
        tvReg::tvLM(fit$formula,
                    data = fit$data[train_idx, ],
                    bw = 0.5),
        error = function(e) NULL)
      if (!is.null(tv_cv)) {
        ntst <- length(test_idx)
        ncoef <- min(ntst, nrow(tv_cv$coefficients))
        pred_f <- rowSums(
          X[test_idx[seq_len(ncoef)], , drop = FALSE] *
          tv_cv$coefficients[seq_len(ncoef), , drop = FALSE])
        if (ncoef < ntst)
          pred_f <- c(pred_f, rep(mean(Y[train_idx]), ntst - ncoef))
      } else {
        pred_f <- rep(mean(Y[train_idx]), length(test_idx))
      }

    } else if (fit$model_type == "Mixed-Effects") {
      me_cv <- tryCatch(
        lme4::lmer(fit$formula_full,
          data = fit$data[train_idx, ],
          REML = FALSE),
        error = function(e) NULL)
      if (!is.null(me_cv))
        pred_f <- predict(me_cv,
          newdata = fit$data[test_idx, ],
          allow.new.levels = TRUE)
      else
        pred_f <- rep(mean(Y[train_idx]), length(test_idx))

    } else {
      fit_cv <- lm(Y[train_idx] ~ X[train_idx, -1])
      pred_f <- predict(fit_cv,
        newdata = data.frame(
          X[test_idx, -1, drop = FALSE]))
    }

    cv_rmse[f] <- sqrt(mean((Y[test_idx] - pred_f)^2))
    cv_mae[f]  <- mean(abs(Y[test_idx] - pred_f))
  }

  # Rolling-window CV
  roll_rmse <- NA
  if (rolling) {
    n0 <- max(20, floor(n * n0_frac))
    if (n0 < n) {
      roll_errs <- numeric(n - n0)
      for (m_idx in n0:(n - 1)) {
        train_idx <- seq_len(m_idx)
        fit_roll <- lm(Y[train_idx] ~ X[train_idx, -1])
        pred_r <- predict(fit_roll,
          newdata = data.frame(
            t(X[m_idx + 1, -1])))
        roll_errs[m_idx - n0 + 1] <- (Y[m_idx + 1] - pred_r)^2
      }
      roll_rmse <- sqrt(mean(roll_errs))
    }
  }

  list(cv_rmse = cv_rmse, cv_mae = cv_mae,
       mean_cv_rmse = mean(cv_rmse),
       mean_cv_mae  = mean(cv_mae),
       roll_rmse = roll_rmse)
}

#' Monte Carlo Forecasting for Arvind Models
#'
#' Generates Monte Carlo forecasts with 80 percent and 95 percent prediction
#' intervals for any fitted \code{ArvindFit} model. Covariates are forecast
#' using SARIMA models (via the \pkg{forecast} package) if not supplied.
#'
#' @param fit an object of class \code{"ArvindFit"}.
#' @param newdata_sims optional named list of pre-computed covariate
#'   simulation matrices, each of dimension \code{h x nsim}.
#' @param h integer; forecast horizon in time steps (default: 120).
#' @param nsim integer; number of Monte Carlo replicates (default: 5000).
#' @param covariate_models optional list of fitted SARIMA models for
#'   covariates (auto-fitted if \code{NULL}).
#' @param seed integer; random seed for reproducibility (default: 123).
#'
#' @return A list with components:
#' \describe{
#'   \item{sims}{matrix (\code{h x nsim}); full simulation matrix.}
#'   \item{mean}{numeric vector length \code{h}; mean forecast.}
#'   \item{median}{numeric vector length \code{h}; median forecast.}
#'   \item{lo80}{numeric vector; lower 80 percent prediction interval.}
#'   \item{hi80}{numeric vector; upper 80 percent prediction interval.}
#'   \item{lo95}{numeric vector; lower 95 percent prediction interval.}
#'   \item{hi95}{numeric vector; upper 95 percent prediction interval.}
#' }
#'
#' @examples
#' \donttest{
#' dat <- simulate_arvind_data(n = 50, seed = 1)
#' m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
#' fc <- forecast_arvind(m1, h = 12, nsim = 100, seed = 42)
#' head(fc$mean)
#' }
#'
#' @seealso [fit_rw1()], [diagnostics_arvind()], [cv_arvind()]
#'
#' @export
forecast_arvind <- function(fit, newdata_sims = NULL,
                            h = 120, nsim = 5000,
                            covariate_models = NULL,
                            seed = 123) {
  set.seed(seed)
  X <- fit$X
  Y <- fit$Y
  n <- fit$n
  p <- fit$p
  theta_m <- fit$theta
  sigma_m <- fit$sigma
  shift_m <- fit$shift

  # Auto-fit SARIMA for covariates if needed
  if (is.null(newdata_sims)) {
    covars <- colnames(X)[-1]
    cov_sims <- list()
    for (col in covars) {
      ts_col <- ts(fit$data[[col]], frequency = 12)
      fit_ar <- forecast::auto.arima(ts_col, seasonal = TRUE)
      sims_mat <- matrix(NA, nrow = h, ncol = nsim)
      for (s in seq_len(nsim))
        sims_mat[, s] <- simulate(fit_ar, nsim = h, future = TRUE)
      cov_sims[[col]] <- sims_mat
    }
    newdata_sims <- cov_sims
  }

  # Monte Carlo forecasting
  Y_fc_sims <- matrix(NA, nrow = h, ncol = nsim)
  covars <- names(newdata_sims)

  for (s in seq_len(nsim)) {
    X_fut <- cbind(1, sapply(covars,
      function(col) newdata_sims[[col]][, s]))
    eps_s <- rarvind(h, theta_m)

    if (fit$model_type == "RW1-approx") {
      beta_fc <- fit$beta_final
      for (tt in seq_len(h)) {
        w_fc <- rarvind_centred(p, fit$theta_innov)
        beta_fc <- beta_fc + fit$sigma_rw * w_fc
        Y_fc_sims[tt, s] <- sum(X_fut[tt, ] * beta_fc) +
          shift_m + sigma_m * eps_s[tt]
      }
    } else if (fit$model_type == "tvLM") {
      beta_last <- fit$tv_coefs[n, ]
      Y_fc_sims[, s] <- as.numeric(X_fut %*% beta_last) +
        shift_m + sigma_m * eps_s

    } else if (fit$model_type == "SIMEX") {
      Y_fc_sims[, s] <- as.numeric(X_fut %*% fit$beta) +
        shift_m + sigma_m * eps_s

    } else if (fit$model_type == "Mixed-Effects") {
      beta_fe <- lme4::fixef(fit$lme_model)
      Y_fc_sims[, s] <- as.numeric(X_fut %*% beta_fe) +
        shift_m + sigma_m * eps_s

    } else if (fit$model_type == "Regime-Switching") {
      tp <- fit$trans_probs
      curr_st <- tail(fit$states, 1)
      for (tt in seq_len(h)) {
        curr_st <- sample(seq_len(fit$nstates), 1,
                          prob = tp[curr_st, ])
        beta_fc <- fit$state_betas[[curr_st]]
        Y_fc_sims[tt, s] <- sum(X_fut[tt, ] * beta_fc) +
          shift_m + sigma_m * eps_s[tt]
      }
    }
  }

  list(
    sims   = Y_fc_sims,
    mean   = rowMeans(Y_fc_sims),
    median = apply(Y_fc_sims, 1, median),
    lo80   = apply(Y_fc_sims, 1, quantile, 0.10),
    hi80   = apply(Y_fc_sims, 1, quantile, 0.90),
    lo95   = apply(Y_fc_sims, 1, quantile, 0.025),
    hi95   = apply(Y_fc_sims, 1, quantile, 0.975)
  )
}

---
title: "Introduction to ArvindSt"
author: "Shikhar Tyagi, Arvind Pandey"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to ArvindSt}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Overview

The **ArvindSt** package provides a unified implementation of the Arvind
distribution and five novel stochastic regression models that replace the
traditional Gaussian error assumption with Arvind-distributed errors.

The Arvind distribution is a flexible single-parameter continuous distribution
on $(0, \infty)$ with PDF:

$$f(x; \theta) = \frac{\theta(1 + 2x + 2\theta x^2)}{(1 + \theta x)^2} \exp(-\theta x^2), \quad x > 0$$

## Distribution Functions

```{r distribution}
library(ArvindSt)

# PDF at several points
darvind(c(0.5, 1, 2), theta = 1)

# CDF
parvind(1, theta = 2)

# Quantiles
qarvind(c(0.25, 0.5, 0.75), theta = 1)

# Random generation
set.seed(42)
x <- rarvind(1000, theta = 2)
summary(x)
```

## Visualising the Arvind Distribution

```{r pdf-plot}
x_seq <- seq(0.01, 4, length.out = 300)
thetas <- c(0.5, 1, 2, 5)

plot(NULL, xlim = c(0, 4), ylim = c(0, 1.5),
     xlab = "x", ylab = "f(x)", main = "Arvind PDF Family")
cols <- c("red", "blue", "darkgreen", "purple")
for (i in seq_along(thetas)) {
  lines(x_seq, darvind(x_seq, thetas[i]), col = cols[i], lwd = 2)
}
legend("topright", paste("theta =", thetas),
       col = cols, lwd = 2, cex = 0.8)
```

## MLE Estimation

```{r mle}
set.seed(42)
x <- rarvind(500, theta = 2)
fit <- fit_arvind_mle(x)
cat("Estimated theta:", fit$theta, "\n")
cat("True theta: 2\n")
```

## Model Fitting Example

```{r model-fit}
# Generate simulated data
dat <- simulate_arvind_data(n = 60, seed = 1)

# Fit RW1 model
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
cat("Model:", m1$model_type, "\n")
cat("Theta:", m1$theta, "\n")
cat("R-squared:", 1 - sum(m1$residuals^2) / sum((m1$Y - mean(m1$Y))^2), "\n")
```

## Diagnostics

```{r diagnostics}
d1 <- diagnostics_arvind(m1)
d1[, c("Model", "RMSE", "R2", "AIC", "KS_pvalue")]
```

## Cross-Validation

```{r cv}
cv1 <- cv_arvind(m1, k_folds = 3, rolling = FALSE, seed = 42)
cat("Mean CV RMSE:", cv1$mean_cv_rmse, "\n")
```

## Summary

The **ArvindSt** package provides:

- Distribution functions: `darvind()`, `parvind()`, `qarvind()`, `rarvind()`
- Simulated data generation: `simulate_arvind_data()`
- Five model-fitting functions: `fit_rw1()`, `fit_tvlm()`, `fit_simex()`,
  `fit_mixed()`, `fit_hmm()`
- Diagnostics: `diagnostics_arvind()`, `plot_arvind()`
- Forecasting: `forecast_arvind()`
- Cross-validation: `cv_arvind()`
- Model comparison: `summary_arvind()`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_fit.R
\name{fit_arvind_mle}
\alias{fit_arvind_mle}
\title{Maximum Likelihood Estimation for the Arvind Distribution}
\usage{
fit_arvind_mle(e_pos)
}
\arguments{
\item{e_pos}{numeric vector of strictly positive observations.}
}
\value{
A list with components:
\describe{
\item{theta}{numeric; the MLE of theta.}
\item{negloglik}{numeric; the minimised negative log-likelihood.}
}
}
\description{
Fits the Arvind distribution to a vector of positive observations by
maximum likelihood. Optimisation is performed on the log-scale via the
Brent method.
}
\examples{
set.seed(42)
x <- rarvind(200, theta = 2)
fit_arvind_mle(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_fit.R
\name{make_arvind_resid}
\alias{make_arvind_resid}
\title{Transform Residuals for Arvind Fitting}
\usage{
make_arvind_resid(resid_raw, Y_ref)
}
\arguments{
\item{resid_raw}{numeric vector of raw residuals.}

\item{Y_ref}{numeric vector of observed response values (used for scaling).}
}
\value{
A list with components:
\describe{
\item{shift}{numeric; the shift applied.}
\item{sigma}{numeric; the standard deviation used for standardisation.}
\item{e_pos}{numeric vector; positive standardised residuals.}
\item{theta}{numeric; MLE of the Arvind parameter.}
\item{negloglik}{numeric; negative log-likelihood at the MLE.}
}
}
\description{
Transforms raw residuals to positive values suitable for fitting the Arvind
distribution by shifting and standardising.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_tvlm.R
\name{fit_tvlm}
\alias{fit_tvlm}
\title{Fit Time-Varying Coefficient Linear Model (tvLM)}
\usage{
fit_tvlm(formula, data, bw = NULL, seed = 42)
}
\arguments{
\item{formula}{an object of class \code{\link[stats]{formula}}.}

\item{data}{a data frame containing the variables in the formula.}

\item{bw}{numeric or \code{NULL}; the bandwidth for kernel smoothing.
If \code{NULL} (default), bandwidth is selected automatically via
leave-one-out cross-validation.}

\item{seed}{integer; random seed for reproducibility (default: 42).}
}
\value{
An object of class \code{"ArvindFit"}, a list containing the
same standard fields as \code{\link[=fit_rw1]{fit_rw1()}}, plus:
\describe{
\item{tv_coefs}{matrix; time-varying coefficient estimates.}
\item{tv_fit}{the fitted \code{tvReg::tvLM} object.}
}
}
\description{
Fits a time-varying coefficient linear model using kernel-weighted
least squares (via the \pkg{tvReg} package) with Arvind-distributed
residuals.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m2 <- fit_tvlm(Y ~ X1 + X2 + X3, dat, bw = 0.5, seed = 42)
m2$theta

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_data.R
\name{simulate_arvind_data}
\alias{simulate_arvind_data}
\title{Generate Simulated Data for Examples}
\usage{
simulate_arvind_data(n = 60, seed = 42)
}
\arguments{
\item{n}{integer; number of observations to generate (default: 60).}

\item{seed}{integer; random seed for reproducibility (default: 42).}
}
\value{
A data frame with columns:
\describe{
\item{Y}{numeric; simulated response variable.}
\item{X1}{numeric; first covariate.}
\item{X2}{numeric; second covariate.}
\item{X3}{numeric; third covariate.}
\item{Group}{factor; grouping variable with 4 levels.}
}
}
\description{
Creates a small simulated dataset that mimics the structure needed for
demonstrating the \pkg{ArvindSt} model-fitting functions. Useful for
examples and testing.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
head(dat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_distribution.R
\name{darvind}
\alias{darvind}
\title{Arvind Distribution Density Function}
\usage{
darvind(x, theta, log = FALSE)
}
\arguments{
\item{x}{numeric vector of quantiles.}

\item{theta}{positive numeric scalar; the distribution parameter.}

\item{log}{logical; if \code{TRUE}, log-density is returned. Default
\code{FALSE}.}
}
\value{
A numeric vector of density values (or log-density values when
\code{log = TRUE}).
}
\description{
Computes the probability density function (PDF) of the Arvind distribution.
}
\details{
The Arvind distribution with parameter \eqn{\theta > 0} has PDF
\deqn{f(x; \theta) = \frac{\theta(1 + 2x + 2\theta x^2)}{(1 + \theta x)^2}
  \exp(-\theta x^2), \quad x > 0.}
}
\examples{
# Evaluate the PDF at several points
darvind(c(0.5, 1, 2), theta = 1)

# Log-density
darvind(1, theta = 2, log = TRUE)

# Returns 0 for x <= 0
darvind(-1, theta = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossval.R
\name{cv_arvind}
\alias{cv_arvind}
\title{K-Fold and Rolling-Window Cross-Validation}
\usage{
cv_arvind(fit, k_folds = 5, rolling = TRUE, n0_frac = 0.5, seed = 42)
}
\arguments{
\item{fit}{an object of class \code{"ArvindFit"}.}

\item{k_folds}{integer; number of cross-validation folds (default: 5).}

\item{rolling}{logical; if \code{TRUE} (default), also performs
rolling-window cross-validation.}

\item{n0_frac}{numeric; fraction of data used as initial training window
for rolling CV (default: 0.5).}

\item{seed}{integer; random seed for reproducibility (default: 42).}
}
\value{
A list with components:
\describe{
\item{cv_rmse}{numeric vector of length \code{k_folds}; per-fold RMSE.}
\item{cv_mae}{numeric vector of length \code{k_folds}; per-fold MAE.}
\item{mean_cv_rmse}{numeric; average k-fold RMSE.}
\item{mean_cv_mae}{numeric; average k-fold MAE.}
\item{roll_rmse}{numeric; rolling-window RMSE (or \code{NA}).}
}
}
\description{
Performs k-fold cross-validation and optionally rolling-window
(expanding-window) cross-validation for an \code{ArvindFit} model.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
cv <- cv_arvind(m1, k_folds = 3, rolling = FALSE, seed = 42)
cv$mean_cv_rmse

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast_arvind}
\alias{forecast_arvind}
\title{Monte Carlo Forecasting for Arvind Models}
\usage{
forecast_arvind(
  fit,
  newdata_sims = NULL,
  h = 120,
  nsim = 5000,
  covariate_models = NULL,
  seed = 123
)
}
\arguments{
\item{fit}{an object of class \code{"ArvindFit"}.}

\item{newdata_sims}{optional named list of pre-computed covariate
simulation matrices, each of dimension \code{h x nsim}.}

\item{h}{integer; forecast horizon in time steps (default: 120).}

\item{nsim}{integer; number of Monte Carlo replicates (default: 5000).}

\item{covariate_models}{optional list of fitted SARIMA models for
covariates (auto-fitted if \code{NULL}).}

\item{seed}{integer; random seed for reproducibility (default: 123).}
}
\value{
A list with components:
\describe{
\item{sims}{matrix (\code{h x nsim}); full simulation matrix.}
\item{mean}{numeric vector length \code{h}; mean forecast.}
\item{median}{numeric vector length \code{h}; median forecast.}
\item{lo80}{numeric vector; lower 80 percent prediction interval.}
\item{hi80}{numeric vector; upper 80 percent prediction interval.}
\item{lo95}{numeric vector; lower 95 percent prediction interval.}
\item{hi95}{numeric vector; upper 95 percent prediction interval.}
}
}
\description{
Generates Monte Carlo forecasts with 80 percent and 95 percent prediction
intervals for any fitted \code{ArvindFit} model. Covariates are forecast
using SARIMA models (via the \pkg{forecast} package) if not supplied.
}
\examples{
\donttest{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
fc <- forecast_arvind(m1, h = 12, nsim = 100, seed = 42)
head(fc$mean)
}

}
\seealso{
\code{\link[=fit_rw1]{fit_rw1()}}, \code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ArvindSt-package.R
\docType{package}
\name{ArvindSt-package}
\alias{ArvindSt}
\alias{ArvindSt-package}
\title{ArvindSt: Five Novel Stochastic Regression Models with Arvind-Distributed Errors and Effects}
\description{
Implements the 'Arvind' distribution and five novel stochastic regression models that replace the traditional Gaussian error assumption with 'Arvind'-distributed errors. The 'Arvind' distribution is a flexible single-parameter continuous distribution on the positive real line characterised by a polynomial numerator with Gaussian-type decay. The package provides complete distribution functions (darvind(), parvind(), qarvind(), rarvind()), maximum likelihood estimation via fit_arvind_mle(), and five model-fitting routines: Random Walk on Coefficients via fit_rw1(), Time-Varying Coefficient Linear Model via fit_tvlm(), Simulation-Extrapolation via fit_simex(), Mixed-Effects Regression via fit_mixed(), and Regime-Switching Hidden Markov Model via fit_hmm(). Additionally provides Monte Carlo forecasting with prediction intervals via forecast_arvind(), comprehensive goodness-of-fit diagnostics (21 metrics and 25 plots) via diagnostics_arvind() and plot_arvind(), k-fold and rolling-window cross-validation via cv_arvind(), and unified model comparison via summary_arvind(). For more details see Pandey, Singh, Tyagi, and Tyagi (2024), "Modelling climate, COVID-19, and reliability data: A new continuous lifetime model under different methods of estimation", 'Statistics and Applications', 22(2).
}
\references{
Pandey, A., Singh, R.P., Tyagi, S., and Tyagi, A. (2024). Modelling climate, COVID-19, and reliability data: A new continuous lifetime model under different methods of estimation. \emph{Statistics and Applications}, 22(2).
}
\author{
\strong{Maintainer}: Shikhar Tyagi \email{shikhar1093tyagi@gmail.com} (\href{https://orcid.org/0000-0003-1606-0844}{ORCID})

Authors:
\itemize{
  \item Arvind Pandey \email{arvindmzu@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_arvind}
\alias{plot_arvind}
\title{Diagnostic Plots for Arvind Models}
\usage{
plot_arvind(fit, output_dir = tempdir(), prefix = NULL)
}
\arguments{
\item{fit}{an object of class \code{"ArvindFit"}.}

\item{output_dir}{character; directory where plots are saved. Defaults to
a temporary directory.}

\item{prefix}{character or \code{NULL}; prefix for plot filenames. If
\code{NULL}, derived from the model type.}
}
\value{
The \code{fit} object is returned invisibly.
}
\description{
Generates up to 25 diagnostic plots for a fitted \code{ArvindFit} object,
including observed vs fitted, residual histogram with Arvind density overlay,
Q-Q plot, ACF, ECDF comparison, and more.
}
\examples{
\donttest{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
plot_arvind(m1, output_dir = tempdir())
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{diagnostics_arvind}
\alias{diagnostics_arvind}
\title{Goodness-of-Fit Diagnostics for Arvind Models}
\usage{
diagnostics_arvind(fit)
}
\arguments{
\item{fit}{an object of class \code{"ArvindFit"} returned by any of the
model-fitting functions.}
}
\value{
A data frame with one row and 21 columns of diagnostics metrics.
See Details for the full list.
}
\description{
Computes 21 goodness-of-fit metrics for any fitted \code{ArvindFit}
object, including MSE, RMSE, MAE, MAPE, R-squared, AIC, BIC,
Kolmogorov-Smirnov test, Anderson-Darling statistic, and more.
}
\details{
The following metrics are computed:
\describe{
\item{Model}{character; the model type.}
\item{MSE}{Mean Squared Error.}
\item{RMSE}{Root Mean Squared Error.}
\item{MAE}{Mean Absolute Error.}
\item{MAPE}{Mean Absolute Percentage Error.}
\item{R2}{R-squared.}
\item{AdjR2}{Adjusted R-squared.}
\item{AIC}{Akaike Information Criterion.}
\item{AICc}{Corrected AIC.}
\item{BIC}{Bayesian Information Criterion.}
\item{LogLik}{Log-likelihood at the MLE.}
\item{Bias}{Mean residual.}
\item{MASE}{Mean Absolute Scaled Error.}
\item{DW}{Durbin-Watson statistic.}
\item{LjungBox_stat}{Ljung-Box test statistic.}
\item{LjungBox_p}{Ljung-Box p-value.}
\item{Theta}{Estimated Arvind parameter.}
\item{KS_stat}{Kolmogorov-Smirnov test statistic.}
\item{KS_pvalue}{Kolmogorov-Smirnov p-value.}
\item{AD_stat}{Anderson-Darling test statistic.}
\item{CvM_stat}{Cramer-von Mises test statistic.}
}
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
diagnostics_arvind(m1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_distribution.R
\name{rarvind_centred}
\alias{rarvind_centred}
\title{Centred Random Generation from the Arvind Distribution}
\usage{
rarvind_centred(n, theta)
}
\arguments{
\item{n}{positive integer; number of random variates to generate.}

\item{theta}{positive numeric scalar; the distribution parameter.}
}
\value{
A numeric vector of length \code{n} with approximately zero mean.
}
\description{
Generates centred Arvind variates with approximately zero mean, suitable
for use as error terms and innovation terms in stochastic regression models.
}
\details{
The centred variate is computed as \eqn{\tilde{\varepsilon} =
\varepsilon - \mu_A(\theta)}, where \eqn{\varepsilon \sim
\mathrm{Arvind}(\theta)} and \eqn{\mu_A(\theta)} is the mean of the
Arvind distribution.
}
\examples{
set.seed(42)
eps <- rarvind_centred(1000, theta = 2)
mean(eps)  # approximately 0

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_fit.R
\name{arvind_mean_fn}
\alias{arvind_mean_fn}
\title{Mean of the Arvind Distribution}
\usage{
arvind_mean_fn(theta)
}
\arguments{
\item{theta}{positive numeric scalar; the distribution parameter.}
}
\value{
A numeric scalar giving the theoretical mean, or \code{NA} if
integration fails.
}
\description{
Computes the theoretical mean of the Arvind distribution with parameter
\code{theta} by numerical integration.
}
\examples{
arvind_mean_fn(1)
arvind_mean_fn(2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_distribution.R
\name{rarvind}
\alias{rarvind}
\title{Random Generation from the Arvind Distribution}
\usage{
rarvind(n, theta)
}
\arguments{
\item{n}{positive integer; number of random variates to generate.}

\item{theta}{positive numeric scalar; the distribution parameter.}
}
\value{
A numeric vector of length \code{n} containing positive random
variates.
}
\description{
Generates random variates from the Arvind distribution using a rejection
sampling algorithm with a half-normal proposal distribution.
}
\examples{
set.seed(42)
x <- rarvind(100, theta = 1)
summary(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_mixed.R
\name{fit_mixed}
\alias{fit_mixed}
\title{Fit Mixed-Effects Regression with Arvind Errors}
\usage{
fit_mixed(formula, data, group_var = "Season", re_formula = NULL, seed = 42)
}
\arguments{
\item{formula}{an object of class \code{\link[stats]{formula}} specifying
the fixed-effects structure.}

\item{data}{a data frame containing the variables in the formula and the
grouping variable.}

\item{group_var}{character string; the name of the grouping variable in
\code{data} (default: \code{"Season"}).}

\item{re_formula}{optional random-effects formula (e.g.,
\code{(1 + X1 | group)}). If \code{NULL} (default), a random intercept
model \code{(1 | group_var)} is used.}

\item{seed}{integer; random seed for reproducibility (default: 42).}
}
\value{
An object of class \code{"ArvindFit"}, a list containing the
same standard fields as \code{\link[=fit_rw1]{fit_rw1()}}, plus:
\describe{
\item{lme_model}{the fitted \code{lme4::lmer} object.}
\item{theta_re}{numeric; Arvind parameter estimated from random effects.}
\item{group_var}{character; the grouping variable name.}
}
}
\description{
Fits a mixed-effects regression model with Arvind-distributed random
effects and observation-level errors. Estimation uses a two-stage approach:
REML initialisation via \pkg{lme4}, followed by Arvind MLE on the
residuals.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m4 <- fit_mixed(Y ~ X1 + X2 + X3, dat, group_var = "Group", seed = 42)
m4$theta
m4$theta

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_distribution.R
\name{parvind}
\alias{parvind}
\title{Arvind Distribution Function (CDF)}
\usage{
parvind(q, theta, lower.tail = TRUE)
}
\arguments{
\item{q}{numeric vector of quantiles.}

\item{theta}{positive numeric scalar; the distribution parameter.}

\item{lower.tail}{logical; if \code{TRUE} (default), probabilities are
\eqn{P(X \le q)}{P(X <= q)}; otherwise \eqn{P(X > q)}.}
}
\value{
A numeric vector of probabilities.
}
\description{
Computes the cumulative distribution function (CDF) of the Arvind
distribution.
}
\details{
The CDF is given by
\deqn{F(x; \theta) = 1 - \frac{1}{1 + \theta x} \exp(-\theta x^2),
  \quad x > 0.}
}
\examples{
parvind(1, theta = 1)
parvind(c(0.5, 1, 2), theta = 2)
parvind(1, theta = 1, lower.tail = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_rw1.R
\name{fit_rw1}
\alias{fit_rw1}
\title{Fit Random Walk on Coefficients Model (RW1-approx)}
\usage{
fit_rw1(formula, data, theta_innov = 2, rw_scale = 0.01, seed = 42)
}
\arguments{
\item{formula}{an object of class \code{\link[stats]{formula}} specifying
the model (e.g., \code{Y ~ X1 + X2}).}

\item{data}{a data frame containing the variables in the formula.}

\item{theta_innov}{positive numeric; the Arvind parameter for state
innovations (default: 2.0).}

\item{rw_scale}{numeric; proportion of OLS coefficients used as innovation
scale (default: 0.01).}

\item{seed}{integer; random seed for reproducibility (default: 42).}
}
\value{
An object of class \code{"ArvindFit"}, a list containing:
\describe{
\item{model_type}{character; \code{"RW1-approx"}.}
\item{fitted}{numeric vector; fitted values.}
\item{residuals}{numeric vector; raw residuals.}
\item{theta}{numeric; estimated Arvind parameter for residuals.}
\item{sigma}{numeric; residual scale.}
\item{shift}{numeric; shift applied to residuals.}
\item{e_pos}{numeric vector; positive standardised residuals.}
\item{negloglik}{numeric; negative log-likelihood.}
\item{beta_t}{matrix; time-varying coefficient paths.}
\item{beta_final}{numeric vector; final coefficient values.}
\item{sigma_rw}{numeric vector; random walk innovation scales.}
\item{theta_innov}{numeric; Arvind parameter used for innovations.}
\item{n}{integer; number of observations.}
\item{p}{integer; number of parameters.}
\item{X}{matrix; design matrix.}
\item{Y}{numeric vector; response variable.}
\item{formula}{the model formula.}
\item{data}{the input data frame.}
}
}
\description{
Fits a stochastic regression model with time-varying coefficients evolving
as a random walk with Arvind-distributed innovations. The observation
errors also follow the Arvind distribution.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
m1$theta

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_hmm.R
\name{fit_hmm}
\alias{fit_hmm}
\title{Fit Regime-Switching Regression (HMM)}
\usage{
fit_hmm(formula, data, nstates = 2, seed = 42, maxiter = 100, tol = 1e-6)
}
\arguments{
\item{formula}{an object of class \code{\link[stats]{formula}}.}

\item{data}{a data frame containing the variables in the formula.}

\item{nstates}{integer; number of hidden states (default: 2).}

\item{seed}{integer; random seed for reproducibility (default: 42).}

\item{maxiter}{integer; maximum EM iterations (default: 100).}

\item{tol}{numeric; convergence tolerance (default: 1e-6).}
}
\value{
An object of class \code{"ArvindFit"}, a list containing the
same standard fields as \code{\link[=fit_rw1]{fit_rw1()}}, plus:
\describe{
\item{nstates}{integer; number of hidden states.}
\item{states}{integer vector; Viterbi-decoded state sequence.}
\item{trans_probs}{matrix; estimated transition probability matrix.}
\item{state_betas}{list of numeric vectors; state-specific coefficients.}
\item{state_sigmas}{numeric vector; state-specific standard deviations.}
}
}
\description{
Fits a hidden Markov model with state-dependent coefficients and
Arvind-distributed errors. The EM algorithm with forward-backward
recursions is used for parameter estimation, and the Viterbi algorithm
decodes the most likely state sequence.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m5 <- fit_hmm(Y ~ X1 + X2 + X3, dat, nstates = 2, seed = 42)
m5$states

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arvind_distribution.R
\name{qarvind}
\alias{qarvind}
\title{Arvind Distribution Quantile Function}
\usage{
qarvind(p, theta)
}
\arguments{
\item{p}{numeric vector of probabilities (\eqn{0 \le p \le 1}).}

\item{theta}{positive numeric scalar; the distribution parameter.}
}
\value{
A numeric vector of quantiles.
}
\description{
Computes quantiles of the Arvind distribution by numerical inversion
of the CDF using \code{\link[stats]{uniroot}}.
}
\examples{
qarvind(0.5, theta = 1)
qarvind(c(0.25, 0.5, 0.75), theta = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary_arvind}
\alias{summary_arvind}
\title{Summary and Comparison of Multiple Arvind Models}
\usage{
summary_arvind(..., comparison_plots = TRUE, output_dir = tempdir())
}
\arguments{
\item{...}{one or more objects of class \code{"ArvindFit"}.}

\item{comparison_plots}{logical; if \code{TRUE} (default), generate
comparison plots.}

\item{output_dir}{character; directory to save comparison plots.
Defaults to a temporary directory.}
}
\value{
A data frame of diagnostic metrics (one row per model) is returned
invisibly.
}
\description{
Accepts multiple \code{ArvindFit} objects, computes diagnostics for each,
produces a unified comparison table, and prints the best model by RMSE,
R-squared, and AIC.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m1 <- fit_rw1(Y ~ X1 + X2 + X3, dat, seed = 42)
summary_arvind(m1)

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=plot_arvind]{plot_arvind()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model_simex.R
\name{fit_simex}
\alias{fit_simex}
\title{Fit Simulation-Extrapolation (SIMEX) Model}
\usage{
fit_simex(
  formula,
  data,
  me_vars = NULL,
  me_frac = 0.05,
  lambda_grid = c(0.5, 1, 1.5, 2),
  n_sim = 100,
  theta_me = 2,
  seed = 123
)
}
\arguments{
\item{formula}{an object of class \code{\link[stats]{formula}}.}

\item{data}{a data frame containing the variables in the formula.}

\item{me_vars}{character vector of covariate names measured with error.
If \code{NULL} (default), the first two term labels are used.}

\item{me_frac}{numeric; fraction of marginal variance used as measurement
error variance (default: 0.05).}

\item{lambda_grid}{numeric vector; SIMEX lambda grid
(default: \code{c(0.5, 1, 1.5, 2)}).}

\item{n_sim}{integer; number of SIMEX simulation replicates (default: 100).}

\item{theta_me}{positive numeric; Arvind parameter for measurement error
(default: 2.0).}

\item{seed}{integer; random seed for reproducibility (default: 123).}
}
\value{
An object of class \code{"ArvindFit"}, a list containing the
same standard fields as \code{\link[=fit_rw1]{fit_rw1()}}, plus:
\describe{
\item{beta}{numeric vector; SIMEX-corrected coefficient estimates.}
\item{simex_coefs}{matrix; coefficient estimates at each lambda level.}
\item{lambda_grid}{numeric vector; the SIMEX lambda grid used.}
\item{me_vars}{character vector; covariate names with measurement error.}
\item{sigma2_me}{named numeric vector; measurement error variances.}
}
}
\description{
Fits a regression model correcting for measurement error attenuation using
the SIMEX algorithm with Arvind-distributed measurement noise and residuals.
}
\examples{
dat <- simulate_arvind_data(n = 50, seed = 1)
m3 <- fit_simex(Y ~ X1 + X2 + X3, dat,
                me_vars = c("X1", "X2"),
                n_sim = 20, seed = 123)
m3$beta
m3$beta

}
\seealso{
\code{\link[=diagnostics_arvind]{diagnostics_arvind()}}, \code{\link[=forecast_arvind]{forecast_arvind()}}, \code{\link[=cv_arvind]{cv_arvind()}}
}
